"""List of languages and their language codes.

The language codes contain both all the ISO 639-1 codes, as well as the ISO 639-3 codes
for languages that do not have an ISO 639-1 code.
"""

from dataclasses import dataclass, field


@dataclass
class Language:
    """A benchmarkable language.

    Attributes:
        code:
            The ISO 639-1 language code of the language.
        name:
            The name of the language.
        and_separator (optional):
            The word 'and' in the language.
        or_separator (optional):
            The word 'or' in the language.
    """

    code: str
    name: str
    _and_separator: str | None = field(repr=False, default=None)
    _or_separator: str | None = field(repr=False, default=None)

    def __hash__(self) -> int:
        """Return a hash of the language."""
        return hash(self.code)

    @property
    def and_separator(self) -> str:
        """Get the word 'and' in the language.

        Returns:
            The word 'and' in the language.

        Raises:
            NotImplementedError:
                If `and_separator` is `None`.
        """
        if not self._and_separator:
            raise NotImplementedError(
                f"Separator for the word 'and' has not been defined for {self.name}."
            )
        return self._and_separator

    @and_separator.setter
    def and_separator(self, value: str | None) -> None:
        self._and_separator = value

    @property
    def or_separator(self) -> str:
        """Get the word 'or' in the language.

        Returns:
            The word 'or' in the language.

        Raises:
            NotImplementedError:
                If `or_separator` is `None`.
        """
        if not self._or_separator:
            raise NotImplementedError(
                f"Separator for the word 'or' has not been defined for {self.name}."
            )
        return self._or_separator

    @or_separator.setter
    def or_separator(self, value: str | None) -> None:
        self._or_separator = value


def get_all_languages() -> dict[str, Language]:
    """Get a list of all the languages.

    Returns:
        A mapping between language codes and their configurations.
    """
    return {cfg.code: cfg for cfg in globals().values() if isinstance(cfg, Language)}


ABKHAZIAN: Language = Language(
    code="ab", name="Abkhazian", _and_separator="и", _or_separator="ма"
)
AFAR: Language = Language(
    code="aa", name="Afar", _and_separator="kee", _or_separator="maleey"
)
AFRIKAANS: Language = Language(
    code="af", name="Afrikaans", _and_separator="en", _or_separator="of"
)
ALBANIAN: Language = Language(
    code="sq", name="Albanian", _and_separator="dhe", _or_separator="ose"
)
AMHARIC: Language = Language(
    code="am", name="Amharic", _and_separator="እና", _or_separator="ወይም"
)
ARABIC: Language = Language(
    code="ar", name="Arabic", _and_separator="و", _or_separator="أو"
)
ARAGONESE: Language = Language(
    code="an", name="Aragonese", _and_separator="y", _or_separator="u"
)
ARMENIAN: Language = Language(
    code="hy", name="Armenian", _and_separator="և", _or_separator="կամ"
)
ASSAMESE: Language = Language(
    code="as", name="Assamese", _and_separator="আৰু", _or_separator="বা"
)
AVARIC: Language = Language(
    code="av", name="Avaric", _and_separator="ги", _or_separator="яги"
)
AVESTAN: Language = Language(
    code="ae", name="Avestan", _and_separator="utā", _or_separator="vā"
)
AYMARA: Language = Language(
    code="ay", name="Aymara", _and_separator="-mpi", _or_separator="jan ukax"
)
AZERBAIJANI: Language = Language(
    code="az", name="Azerbaijani", _and_separator="və", _or_separator="və ya"
)
BAMBARA: Language = Language(
    code="bm", name="Bambara", _and_separator="ani", _or_separator="walima"
)
BASHKIR: Language = Language(
    code="ba", name="Bashkir", _and_separator="һәм", _or_separator="йәки"
)
BASQUE: Language = Language(
    code="eu", name="Basque", _and_separator="eta", _or_separator="edo"
)
BELARUSIAN: Language = Language(
    code="be", name="Belarusian", _and_separator="і", _or_separator="або"
)
BENGALI: Language = Language(
    code="bn", name="Bengali", _and_separator="এবং", _or_separator="অথবা"
)
BISLAMA: Language = Language(
    code="bi", name="Bislama", _and_separator="mo", _or_separator="o"
)
BOSNIAN: Language = Language(
    code="bs", name="Bosnian", _and_separator="i", _or_separator="ili"
)
BRETON: Language = Language(
    code="br", name="Breton", _and_separator="ha", _or_separator="pe"
)
BULGARIAN: Language = Language(
    code="bg", name="Bulgarian", _and_separator="и", _or_separator="или"
)
BURMESE: Language = Language(
    code="my", name="Burmese", _and_separator="နှင့်", _or_separator="သို့မဟုတ်"
)
CATALAN: Language = Language(
    code="ca", name="Catalan", _and_separator="i", _or_separator="o"
)
CHAMORRO: Language = Language(
    code="ch", name="Chamorro", _and_separator="yan", _or_separator="pat"
)
CHECHEN: Language = Language(
    code="ce", name="Chechen", _and_separator="а", _or_separator="я"
)
CHICHEWA: Language = Language(
    code="ny", name="Chichewa", _and_separator="ndi", _or_separator="kapena"
)
SIMPLIFIED_CHINESE: Language = Language(
    code="zh-cn", name="Simplified Chinese", _and_separator="和", _or_separator="或"
)
TRADITIONAL_CHINESE: Language = Language(
    code="zh-tw", name="Traditional Chinese", _and_separator="與", _or_separator="或"
)
CHURCH_SLAVIC: Language = Language(
    code="cu", name="Church Slavic", _and_separator="и", _or_separator="или"
)
CHUVASH: Language = Language(
    code="cv", name="Chuvash", _and_separator="тата", _or_separator="е"
)
CORNISH: Language = Language(
    code="kw", name="Cornish", _and_separator="ha", _or_separator="po"
)
CORSICAN: Language = Language(
    code="co", name="Corsican", _and_separator="e", _or_separator="o"
)
CREE: Language = Language(
    code="cr", name="Cree", _and_separator="ēkwa", _or_separator="kamāc"
)
CROATIAN: Language = Language(
    code="hr", name="Croatian", _and_separator="i", _or_separator="ili"
)
CZECH: Language = Language(
    code="cs", name="Czech", _and_separator="a", _or_separator="nebo"
)
DANISH: Language = Language(
    code="da", name="Danish", _and_separator="og", _or_separator="eller"
)
DUTCH: Language = Language(
    code="nl", name="Dutch", _and_separator="en", _or_separator="of"
)
DIVEHI: Language = Language(
    code="dv", name="Divehi", _and_separator="އަދި", _or_separator="ނުވަތަ"
)
DZONGKHA: Language = Language(
    code="dz", name="Dzongkha", _and_separator="དང་", _or_separator="ཡང་མེན"
)
ENGLISH: Language = Language(
    code="en", name="English", _and_separator="and", _or_separator="or"
)
ESPERANTO: Language = Language(
    code="eo", name="Esperanto", _and_separator="kaj", _or_separator="aŭ"
)
ESTONIAN: Language = Language(
    code="et", name="Estonian", _and_separator="ja", _or_separator="või"
)
EWE: Language = Language(
    code="ee", name="Ewe", _and_separator="kple", _or_separator="alo"
)
FAROESE: Language = Language(
    code="fo", name="Faroese", _and_separator="og", _or_separator="ella"
)
FIJIAN: Language = Language(
    code="fj", name="Fijian", _and_separator="kei", _or_separator="se"
)
FINNISH: Language = Language(
    code="fi", name="Finnish", _and_separator="ja", _or_separator="tai"
)
FRENCH: Language = Language(
    code="fr", name="French", _and_separator="et", _or_separator="ou"
)
WESTERN_FRISIAN: Language = Language(
    code="fy", name="Western Frisian", _and_separator="en", _or_separator="of"
)
FULAH: Language = Language(
    code="ff", name="Fulah", _and_separator="e", _or_separator="ma"
)
GAELIC: Language = Language(
    code="gd", name="Gaelic", _and_separator="agus", _or_separator="no"
)
GALICIAN: Language = Language(
    code="gl", name="Galician", _and_separator="e", _or_separator="ou"
)
GANDA: Language = Language(
    code="lg", name="Ganda", _and_separator="ne", _or_separator="oba"
)
GEORGIAN: Language = Language(
    code="ka", name="Georgian", _and_separator="და", _or_separator="ან"
)
GERMAN: Language = Language(
    code="de", name="German", _and_separator="und", _or_separator="oder"
)
GREEK: Language = Language(
    code="el", name="Greek", _and_separator="και", _or_separator="ή"
)
GREENLANDIC: Language = Language(
    code="kl", name="Greenlandic", _and_separator="aamma", _or_separator="imaluunniit"
)
GUARANI: Language = Language(
    code="gn", name="Guarani", _and_separator="ha", _or_separator="térã"
)
GUJARATI: Language = Language(
    code="gu", name="Gujarati", _and_separator="અને", _or_separator="અથવા"
)
HAITIAN: Language = Language(
    code="ht", name="Haitian", _and_separator="ak", _or_separator="oswa"
)
HAUSA: Language = Language(
    code="ha", name="Hausa", _and_separator="da", _or_separator="ko"
)
HEBREW: Language = Language(
    code="he", name="Hebrew", _and_separator="ו", _or_separator="או"
)
HERERO: Language = Language(
    code="hz", name="Herero", _and_separator="na", _or_separator="po"
)
HINDI: Language = Language(
    code="hi", name="Hindi", _and_separator="और", _or_separator="या"
)
HUNGARIAN: Language = Language(
    code="hu", name="Hungarian", _and_separator="és", _or_separator="vagy"
)
ICELANDIC: Language = Language(
    code="is", name="Icelandic", _and_separator="og", _or_separator="eða"
)
IDO: Language = Language(code="io", name="Ido", _and_separator="e", _or_separator="o")
IGBO: Language = Language(
    code="ig", name="Igbo", _and_separator="na", _or_separator="ma ọ bụ"
)
INDONESIAN: Language = Language(
    code="id", name="Indonesian", _and_separator="dan", _or_separator="atau"
)
INTERLINGUA: Language = Language(
    code="ia", name="Interlingua", _and_separator="e", _or_separator="o"
)
INTERLINGUE: Language = Language(
    code="ie", name="Interlingue", _and_separator="e", _or_separator="o"
)
INUKTITUT: Language = Language(
    code="iu", name="Inuktitut", _and_separator="alu", _or_separator="immaqaa"
)
INUPIAQ: Language = Language(
    code="ik", name="Inupiaq", _and_separator="ġu", _or_separator="luunniit"
)
IRISH: Language = Language(
    code="ga", name="Irish", _and_separator="agus", _or_separator="nó"
)
ITALIAN: Language = Language(
    code="it", name="Italian", _and_separator="e", _or_separator="o"
)
JAPANESE: Language = Language(
    code="ja", name="Japanese", _and_separator="と", _or_separator="または"
)
KANNADA: Language = Language(
    code="kn", name="Kannada", _and_separator="ಮತ್ತು", _or_separator="ಅಥವಾ"
)
KANURI: Language = Language(
    code="kr", name="Kanuri", _and_separator="-a", _or_separator="yáá"
)
KASHMIRI: Language = Language(
    code="ks", name="Kashmiri", _and_separator="تہٕ", _or_separator="یا"
)
KAZAKH: Language = Language(
    code="kk", name="Kazakh", _and_separator="және", _or_separator="немесе"
)
CENTRAL_KHMER: Language = Language(
    code="km", name="Central Khmer", _and_separator="និង", _or_separator="ឬ"
)
KIKUYU: Language = Language(
    code="ki", name="Kikuyu", _and_separator="na", _or_separator="kana"
)
KINYARWANDA: Language = Language(
    code="rw", name="Kinyarwanda", _and_separator="na", _or_separator="cyangwa"
)
KIRGHIZ: Language = Language(
    code="ky", name="Kirghiz", _and_separator="жана", _or_separator="же"
)
KOMI: Language = Language(
    code="kv", name="Komi", _and_separator="да", _or_separator="либӧ"
)
KONGO: Language = Language(
    code="kg", name="Kongo", _and_separator="ye", _or_separator="kana"
)
KOREAN: Language = Language(
    code="ko", name="Korean", _and_separator="그리고", _or_separator="또는"
)
KUANYAMA: Language = Language(
    code="kj", name="Kuanyama", _and_separator="na", _or_separator="nenge"
)
KURDISH: Language = Language(
    code="ku", name="Kurdish", _and_separator="û", _or_separator="an"
)
LAO: Language = Language(code="lo", name="Lao", _and_separator="และ", _or_separator="ຫຼື")
LATIN: Language = Language(
    code="la", name="Latin", _and_separator="et", _or_separator="aut"
)
LATVIAN: Language = Language(
    code="lv", name="Latvian", _and_separator="un", _or_separator="vai"
)
LIMBURGAN: Language = Language(
    code="li", name="Limburgan", _and_separator="en", _or_separator="of"
)
LINGALA: Language = Language(
    code="ln", name="Lingala", _and_separator="na", _or_separator="to"
)
LITHUANIAN: Language = Language(
    code="lt", name="Lithuanian", _and_separator="ir", _or_separator="arba"
)
LUBA_KATANGA: Language = Language(
    code="lu", name="Luba-Katanga", _and_separator="ne", _or_separator="nansha"
)
LUXEMBOURGISH: Language = Language(
    code="lb", name="Luxembourgish", _and_separator="an", _or_separator="oder"
)
MACEDONIAN: Language = Language(
    code="mk", name="Macedonian", _and_separator="и", _or_separator="или"
)
MALAGASY: Language = Language(
    code="mg", name="Malagasy", _and_separator="sy", _or_separator="na"
)
MALAY: Language = Language(
    code="ms", name="Malay", _and_separator="dan", _or_separator="atau"
)
MALAYALAM: Language = Language(
    code="ml", name="Malayalam", _and_separator="ഉം", _or_separator="അല്ലെങ്കിൽ"
)
MALTESE: Language = Language(
    code="mt", name="Maltese", _and_separator="u", _or_separator="jew"
)
MANX: Language = Language(
    code="gv", name="Manx", _and_separator="as", _or_separator="ny"
)
MAORI: Language = Language(
    code="mi", name="Maori", _and_separator="me", _or_separator="rānei"
)
MARATHI: Language = Language(
    code="mr", name="Marathi", _and_separator="आणि", _or_separator="किंवा"
)
MARSHALLESE: Language = Language(
    code="mh", name="Marshallese", _and_separator="im", _or_separator="ak"
)
MONGOLIAN: Language = Language(
    code="mn", name="Mongolian", _and_separator="ба", _or_separator="эсвэл"
)
NAURU: Language = Language(
    code="na", name="Nauru", _and_separator="ma", _or_separator="me"
)
NAVAJO: Language = Language(
    code="nv", name="Navajo", _and_separator="áádóó", _or_separator="doodaiiʼ"
)
NORTHERN_NDEBELE: Language = Language(
    code="nd", name="Northern Ndebele", _and_separator="lo", _or_separator="kumbe"
)
SOUTH_NDEBELE: Language = Language(
    code="nr", name="South Ndebele", _and_separator="na", _or_separator="namkha"
)
NDONGA: Language = Language(
    code="ng", name="Ndonga", _and_separator="na", _or_separator="nenge"
)
NEPALI: Language = Language(
    code="ne", name="Nepali", _and_separator="र", _or_separator="वा"
)
NORWEGIAN: Language = Language(
    code="no", name="Norwegian", _and_separator="og", _or_separator="eller"
)
NORWEGIAN_BOKMÅL: Language = Language(
    code="nb", name="Norwegian Bokmål", _and_separator="og", _or_separator="eller"
)
NORWEGIAN_NYNORSK: Language = Language(
    code="nn", name="Norwegian Nynorsk", _and_separator="og", _or_separator="eller"
)
OCCITAN: Language = Language(
    code="oc", name="Occitan", _and_separator="e", _or_separator="o"
)
OJIBWA: Language = Language(
    code="oj", name="Ojibwa", _and_separator="miinawaa", _or_separator="jiishin"
)
ORIYA: Language = Language(
    code="or", name="Oriya", _and_separator="ଏବଂ", _or_separator="କିମ୍ବା"
)
OROMO: Language = Language(
    code="om", name="Oromo", _and_separator="fi", _or_separator="yookan"
)
OSSETIAN: Language = Language(
    code="os", name="Ossetian", _and_separator="æмæ", _or_separator="кæнæ"
)
PALI: Language = Language(
    code="pi", name="Pali", _and_separator="ca", _or_separator="vā"
)
PASHTO: Language = Language(
    code="ps", name="Pashto", _and_separator="او", _or_separator="يا"
)
PERSIAN: Language = Language(
    code="fa", name="Persian", _and_separator="و", _or_separator="یا"
)
POLISH: Language = Language(
    code="pl", name="Polish", _and_separator="i", _or_separator="lub"
)
PORTUGUESE: Language = Language(
    code="pt", name="Portuguese", _and_separator="e", _or_separator="ou"
)
EUROPEAN_PORTUGUESE: Language = Language(
    code="pt-pt", name="European Portuguese", _and_separator="e", _or_separator="ou"
)
BRAZILIAN_PORTUGUESE: Language = Language(
    code="pt-br", name="Brazilian Portuguese", _and_separator="e", _or_separator="ou"
)
PUNJABI: Language = Language(
    code="pa", name="Punjabi", _and_separator="ਅਤੇ", _or_separator="ਜਾਂ"
)
QUECHUA: Language = Language(
    code="qu", name="Quechua", _and_separator="-pas", _or_separator="ichataq"
)
ROMANIAN: Language = Language(
    code="ro", name="Romanian", _and_separator="și", _or_separator="sau"
)
ROMANSH: Language = Language(
    code="rm", name="Romansh", _and_separator="e", _or_separator="u"
)
RUNDI: Language = Language(
    code="rn", name="Rundi", _and_separator="na", _or_separator="canke"
)
RUSSIAN: Language = Language(
    code="ru", name="Russian", _and_separator="и", _or_separator="или"
)
NORTHERN_SAMI: Language = Language(
    code="se", name="Northern Sami", _and_separator="ja", _or_separator="dahje"
)
SAMOAN: Language = Language(
    code="sm", name="Samoan", _and_separator="ma", _or_separator="poʻo"
)
SANGO: Language = Language(
    code="sg", name="Sango", _and_separator="na", _or_separator="wala"
)
SANSKRIT: Language = Language(
    code="sa", name="Sanskrit", _and_separator="च", _or_separator="वा"
)
SARDINIAN: Language = Language(
    code="sc", name="Sardinian", _and_separator="e", _or_separator="o"
)
SERBIAN: Language = Language(
    code="sr", name="Serbian", _and_separator="и", _or_separator="или"
)
SHONA: Language = Language(
    code="sn", name="Shona", _and_separator="uye", _or_separator="kana"
)
SINDHI: Language = Language(
    code="sd", name="Sindhi", _and_separator="۽", _or_separator="يا"
)
SINHALA: Language = Language(
    code="si", name="Sinhala", _and_separator="සහ", _or_separator="හෝ"
)
SLOVAK: Language = Language(
    code="sk", name="Slovak", _and_separator="a", _or_separator="alebo"
)
SLOVENE: Language = Language(
    code="sl", name="Slovene", _and_separator="in", _or_separator="ali"
)
SOMALI: Language = Language(
    code="so", name="Somali", _and_separator="iyo", _or_separator="ama"
)
SOTHO: Language = Language(
    code="st", name="Sotho", _and_separator="le", _or_separator="kapa"
)
SPANISH: Language = Language(
    code="es", name="Spanish", _and_separator="y", _or_separator="o"
)
SUNDANESE: Language = Language(
    code="su", name="Sundanese", _and_separator="jeung", _or_separator="atawa"
)
SWAHILI: Language = Language(
    code="sw", name="Swahili", _and_separator="na", _or_separator="au"
)
SWATI: Language = Language(
    code="ss", name="Swati", _and_separator="na", _or_separator="noma"
)
SWEDISH: Language = Language(
    code="sv", name="Swedish", _and_separator="och", _or_separator="eller"
)
TAGALOG: Language = Language(
    code="tl", name="Tagalog", _and_separator="at", _or_separator="o"
)
TAHITIAN: Language = Language(
    code="ty", name="Tahitian", _and_separator="e", _or_separator="aore ra"
)
TAJIK: Language = Language(
    code="tg", name="Tajik", _and_separator="ва", _or_separator="ё"
)
TAMIL: Language = Language(
    code="ta", name="Tamil", _and_separator="மற்றும்", _or_separator="அல்லது"
)
TATAR: Language = Language(
    code="tt", name="Tatar", _and_separator="һәм", _or_separator="яки"
)
TELUGU: Language = Language(
    code="te", name="Telugu", _and_separator="మరియు", _or_separator="లేదా"
)
THAI: Language = Language(
    code="th", name="Thai", _and_separator="และ", _or_separator="หรือ"
)
TIBETAN: Language = Language(
    code="bo", name="Tibetan", _and_separator="དང་", _or_separator="ཡང་ན"
)
TIGRINYA: Language = Language(
    code="ti", name="Tigrinya", _and_separator="ን", _or_separator="ወይ"
)
TONGA: Language = Language(
    code="to", name="Tonga", _and_separator="mo", _or_separator="pe"
)
TSONGA: Language = Language(
    code="ts", name="Tsonga", _and_separator="na", _or_separator="kumbe"
)
TSWANA: Language = Language(
    code="tn", name="Tswana", _and_separator="le", _or_separator="kgotsa"
)
TURKISH: Language = Language(
    code="tr", name="Turkish", _and_separator="ve", _or_separator="veya"
)
TURKMEN: Language = Language(
    code="tk", name="Turkmen", _and_separator="we", _or_separator="ýa-da"
)
TWI: Language = Language(
    code="tw", name="Twi", _and_separator="ne", _or_separator="anaa"
)
UIGHUR: Language = Language(
    code="ug", name="Uighur", _and_separator="ۋە", _or_separator="ياكى"
)
UKRAINIAN: Language = Language(
    code="uk", name="Ukrainian", _and_separator="і", _or_separator="або"
)
URDU: Language = Language(
    code="ur", name="Urdu", _and_separator="اور", _or_separator="یا"
)
UZBEK: Language = Language(
    code="uz", name="Uzbek", _and_separator="va", _or_separator="yoki"
)
VENDA: Language = Language(
    code="ve", name="Venda", _and_separator="na", _or_separator="kana"
)
VIETNAMESE: Language = Language(
    code="vi", name="Vietnamese", _and_separator="và", _or_separator="hoặc"
)
VOLAPÜK: Language = Language(
    code="vo", name="Volapük", _and_separator="e", _or_separator="u"
)
WALLOON: Language = Language(
    code="wa", name="Walloon", _and_separator="et", _or_separator="ou"
)
WELSH: Language = Language(
    code="cy", name="Welsh", _and_separator="a", _or_separator="neu"
)
WOLOF: Language = Language(
    code="wo", name="Wolof", _and_separator="ak", _or_separator="walla"
)
XHOSA: Language = Language(
    code="xh", name="Xhosa", _and_separator="kwaye", _or_separator="okanye"
)
YIDDISH: Language = Language(
    code="yi", name="Yiddish", _and_separator="און", _or_separator="אָדער"
)
YORUBA: Language = Language(
    code="yo", name="Yoruba", _and_separator="àti", _or_separator="tàbí"
)
ZHUANG: Language = Language(
    code="za", name="Zhuang", _and_separator="kae", _or_separator="aevih"
)
ZULU: Language = Language(
    code="zu", name="Zulu", _and_separator="futhi", _or_separator="noma"
)
ACEHNESE: Language = Language(
    code="ace", name="Acehnese", _and_separator="ngon", _or_separator="atɔ"
)
ADYGHE: Language = Language(
    code="ady", name="Adyghe", _and_separator="рэ", _or_separator="е"
)
SOUTHERN_ALTAI: Language = Language(
    code="alt", name="Southern Altai", _and_separator="ла", _or_separator="эмезе"
)
AMIS: Language = Language(
    code="ami", name="Amis", _and_separator="ato", _or_separator="o"
)
OLD_ENGLISH: Language = Language(
    code="ang", name="Old English", _and_separator="and", _or_separator="oþþe"
)
ANGIKA: Language = Language(
    code="anp", name="Angika", _and_separator="आर", _or_separator="या"
)
ARAMAIC: Language = Language(
    code="arc", name="Aramaic", _and_separator="ܘ", _or_separator="ܐܘ"
)
MOROCCAN_ARABIC: Language = Language(
    code="ary", name="Moroccan Arabic", _and_separator="w", _or_separator="wella"
)
EGYPTIAN_ARABIC: Language = Language(
    code="arz", name="Egyptian Arabic", _and_separator="و", _or_separator="أو"
)
ASTURIAN: Language = Language(
    code="ast", name="Asturian", _and_separator="y", _or_separator="o"
)
ATIKAMEKW: Language = Language(
    code="atj", name="Atikamekw", _and_separator="et", _or_separator="ou"
)
KOTAVA: Language = Language(
    code="avk", name="Kotava", _and_separator="is", _or_separator="en"
)
AWADHI: Language = Language(
    code="awa", name="Awadhi", _and_separator="अउ", _or_separator="या"
)
SOUTH_AZERBAIJANI: Language = Language(
    code="azb", name="South Azerbaijani", _and_separator="و", _or_separator="یوخسا"
)
BALINESE: Language = Language(
    code="ban", name="Balinese", _and_separator="lan", _or_separator="utawi"
)
BAVARIAN: Language = Language(
    code="bar", name="Bavarian", _and_separator="und", _or_separator="oda"
)
CENTRAL_BIKOL: Language = Language(
    code="bcl", name="Central Bikol", _and_separator="asin", _or_separator="o"
)
BANJAR: Language = Language(
    code="bjn", name="Banjar", _and_separator="wan", _or_separator="atawa"
)
PAO: Language = Language(
    code="blk", name="Pa'O", _and_separator="နန်", _or_separator="မု"
)
BISHNUPRIYA: Language = Language(
    code="bpy", name="Bishnupriya", _and_separator="आ", _or_separator="বা"
)
BUGINESE: Language = Language(
    code="bug", name="Buginese", _and_separator="na", _or_separator="iyarega"
)
BURIAT: Language = Language(
    code="bxr", name="Buriat", _and_separator="ба", _or_separator="али"
)
MINDONG_CHINESE: Language = Language(
    code="cdo", name="Mindong Chinese", _and_separator="共", _or_separator="或者"
)
CEBUANO: Language = Language(
    code="ceb", name="Cebuano", _and_separator="ug", _or_separator="o"
)
CHEROKEE: Language = Language(
    code="chr", name="Cherokee", _and_separator="ᎠᎴ", _or_separator="ᎠᎴ"
)
CHEYENNE: Language = Language(
    code="chy", name="Cheyenne", _and_separator="na", _or_separator="hēme"
)
CENTRAL_KURDISH: Language = Language(
    code="ckb", name="Central Kurdish", _and_separator="و", _or_separator="یان"
)
CRIMEAN_TATAR: Language = Language(
    code="crh", name="Crimean Tatar", _and_separator="ve", _or_separator="ya da"
)
KASHUBIAN: Language = Language(
    code="csb", name="Kashubian", _and_separator="ë", _or_separator="abò"
)
DAGBANI: Language = Language(
    code="dag", name="Dagbani", _and_separator="n-ti", _or_separator="bee"
)
DINKA: Language = Language(
    code="din", name="Dinka", _and_separator="ka", _or_separator="ke"
)
DIMLI: Language = Language(
    code="diq", name="Dimli", _and_separator="û", _or_separator="ya"
)
LOWER_SORBIAN: Language = Language(
    code="dsb", name="Lower Sorbian", _and_separator="a", _or_separator="abo"
)
DOTELI: Language = Language(
    code="dty", name="Doteli", _and_separator="र", _or_separator="या"
)
EXTREMADURAN: Language = Language(
    code="ext", name="Extremaduran", _and_separator="y", _or_separator="u"
)
FANTI: Language = Language(
    code="fat", name="Fanti", _and_separator="na", _or_separator="anaa"
)
FON: Language = Language(
    code="fon", name="Fon", _and_separator="kpóɖó", _or_separator="kabi"
)
ARPITAN: Language = Language(
    code="frp", name="Arpitan", _and_separator="et", _or_separator="ou"
)
NORTHERN_FRISIAN: Language = Language(
    code="frr", name="Northern Frisian", _and_separator="an", _or_separator="of"
)
FRIULIAN: Language = Language(
    code="fur", name="Friulian", _and_separator="e", _or_separator="o"
)
GAGAUZ: Language = Language(
    code="gag", name="Gagauz", _and_separator="hem", _or_separator="ya"
)
GAN_CHINESE: Language = Language(
    code="gan", name="Gan Chinese", _and_separator="同", _or_separator="或"
)
GUIANAN_CREOLE: Language = Language(
    code="gcr", name="Guianan Creole", _and_separator="ké", _or_separator="ou"
)
GILAKI: Language = Language(
    code="glk", name="Gilaki", _and_separator="و", _or_separator="یا"
)
GOAN_KONKANI: Language = Language(
    code="gom", name="Goan Konkani", _and_separator="आनी", _or_separator="वा"
)
GORONTALO: Language = Language(
    code="gor", name="Gorontalo", _and_separator="wawu", _or_separator="meyalo"
)
GOTHIC: Language = Language(
    code="got", name="Gothic", _and_separator="jah", _or_separator="aiþþau"
)
GHANAIAN_PIDGIN: Language = Language(
    code="gpe", name="Ghanaian Pidgin", _and_separator="and", _or_separator="anaa"
)
WAYUU: Language = Language(
    code="guc", name="Wayuu", _and_separator="je", _or_separator="yaa"
)
FRAFRA: Language = Language(
    code="gur", name="Frafra", _and_separator="la", _or_separator="bee"
)
GUN: Language = Language(
    code="guw", name="Gun", _and_separator="pódó", _or_separator="yèkì"
)
HAKKA_CHINESE: Language = Language(
    code="hak", name="Hakka Chinese", _and_separator="同", _or_separator="或者"
)
HAWAIIAN: Language = Language(
    code="haw", name="Hawaiian", _and_separator="a", _or_separator="a i ʻole"
)
FIJI_HINDI: Language = Language(
    code="hif", name="Fiji Hindi", _and_separator="aur", _or_separator="ki"
)
UPPER_SORBIAN: Language = Language(
    code="hsb", name="Upper Sorbian", _and_separator="a", _or_separator="abo"
)
WESTERN_ARMENIAN: Language = Language(
    code="hyw", name="Western Armenian", _and_separator="եւ", _or_separator="կամ"
)
ILOKO: Language = Language(
    code="ilo", name="Iloko", _and_separator="ken", _or_separator="wenno"
)
INGUSH: Language = Language(
    code="inh", name="Ingush", _and_separator="и", _or_separator="е"
)
JAMAICAN_CREOLE: Language = Language(
    code="jam", name="Jamaican Creole", _and_separator="an", _or_separator="ar"
)
LOJBAN: Language = Language(
    code="jbo", name="Lojban", _and_separator="e", _or_separator="a"
)
KARA_KALPAK: Language = Language(
    code="kaa", name="Kara-Kalpak", _and_separator="ha'm", _or_separator="yamasa"
)
KABYLE: Language = Language(
    code="kab", name="Kabyle", _and_separator="d", _or_separator="neɣ"
)
KABARDIAN: Language = Language(
    code="kbd", name="Kabardian", _and_separator="рэ", _or_separator="хэтӀэ"
)
KABIYÈ: Language = Language(
    code="kbp", name="Kabiyè", _and_separator="nɛ", _or_separator="yaa"
)
TYAP: Language = Language(
    code="kcg", name="Tyap", _and_separator="ma", _or_separator="a̠ni"
)
KOMI_PERMYAK: Language = Language(
    code="koi", name="Komi-Permyak", _and_separator="да", _or_separator="либӧ"
)
KARACHAY_BALKAR: Language = Language(
    code="krc", name="Karachay-Balkar", _and_separator="бла", _or_separator="не да"
)
LADINO: Language = Language(
    code="lad", name="Ladino", _and_separator="i", _or_separator="o"
)
LAK: Language = Language(
    code="lbe", name="Lak", _and_separator="ва", _or_separator="ягу"
)
LEZGHIAN: Language = Language(
    code="lez", name="Lezghian", _and_separator="ва", _or_separator="я"
)
LINGUA_FRANCA_NOVA: Language = Language(
    code="lfn", name="Lingua Franca Nova", _and_separator="e", _or_separator="o"
)
LIGURIAN: Language = Language(
    code="lij", name="Ligurian", _and_separator="e", _or_separator="ò"
)
LADIN: Language = Language(
    code="lld", name="Ladin", _and_separator="y", _or_separator="o"
)
LOMBARD: Language = Language(
    code="lmo", name="Lombard", _and_separator="e", _or_separator="o"
)
LATGALIAN: Language = Language(
    code="ltg", name="Latgalian", _and_separator="i", _or_separator="voi"
)
MADURESE: Language = Language(
    code="mad", name="Madurese", _and_separator="ban", _or_separator="o"
)
MAITHILI: Language = Language(
    code="mai", name="Maithili", _and_separator="आ", _or_separator="वा"
)
MOKSHA: Language = Language(
    code="mdf", name="Moksha", _and_separator="ди", _or_separator="или"
)
EASTERN_MARI: Language = Language(
    code="mhr", name="Eastern Mari", _and_separator="да", _or_separator="o"
)
MINANGKABAU: Language = Language(
    code="min", name="Minangkabau", _and_separator="jo", _or_separator="atau"
)
MANIPURI: Language = Language(
    code="mni", name="Manipuri", _and_separator="ꯑꯃꯁꯨꯡ", _or_separator="ꯅꯠꯇ꯭ꯔꯒꯥ"
)
MON: Language = Language(
    code="mnw", name="Mon", _and_separator="ကဵု", _or_separator="ဟွံသေင်မ္ဂး"
)
WESTERN_MARI: Language = Language(
    code="mrj", name="Western Mari", _and_separator="да", _or_separator="o"
)
MIRANDESE: Language = Language(
    code="mwl", name="Mirandese", _and_separator="i", _or_separator="ó"
)
ERZYA: Language = Language(
    code="myv", name="Erzya", _and_separator="ды", _or_separator="эли"
)
MAZANDERANI: Language = Language(
    code="mzn", name="Mazanderani", _and_separator="و", _or_separator="یا"
)
NEAPOLITAN: Language = Language(
    code="nap", name="Neapolitan", _and_separator="e", _or_separator="o"
)
LOW_GERMAN: Language = Language(
    code="nds", name="Low German", _and_separator="un", _or_separator="oder"
)
NEWARI: Language = Language(
    code="new", name="Newari", _and_separator="व", _or_separator="या"
)
NIAS: Language = Language(
    code="nia", name="Nias", _and_separator="ba", _or_separator="mazi"
)
NOVIAL: Language = Language(
    code="nov", name="Novial", _and_separator="e", _or_separator="o"
)
NKO: Language = Language(
    code="nqo", name="N'Ko", _and_separator="ߣߌ߫", _or_separator="ߥߟߊ߫"
)
NORTHERN_SOTHO: Language = Language(
    code="nso", name="Northern Sotho", _and_separator="le", _or_separator="goba"
)
LIVVI_KARELIAN: Language = Language(
    code="olo", name="Livvi-Karelian", _and_separator="da", _or_separator="libo"
)
PANGASINAN: Language = Language(
    code="pag", name="Pangasinan", _and_separator="tan", _or_separator="odino"
)
PAMPANGA: Language = Language(
    code="pam", name="Pampanga", _and_separator="at", _or_separator="o"
)
PAPIAMENTO: Language = Language(
    code="pap", name="Papiamento", _and_separator="y", _or_separator="o"
)
PICARD: Language = Language(
    code="pcd", name="Picard", _and_separator="pi", _or_separator="ou"
)
NIGERIAN_PIDGIN: Language = Language(
    code="pcm", name="Nigerian Pidgin", _and_separator="and", _or_separator="abi"
)
PENNSYLVANIA_GERMAN: Language = Language(
    code="pdc", name="Pennsylvania German", _and_separator="un", _or_separator="odder"
)
PALATINE_GERMAN: Language = Language(
    code="pfl", name="Palatine German", _and_separator="un", _or_separator="odda"
)
PIEMONTESE: Language = Language(
    code="pms", name="Piemontese", _and_separator="e", _or_separator="o"
)
WESTERN_PUNJABI: Language = Language(
    code="pnb", name="Western Punjabi", _and_separator="تے", _or_separator="یا"
)
PONTIC: Language = Language(
    code="pnt", name="Pontic", _and_separator="και", _or_separator="ή"
)
PAIWAN: Language = Language(
    code="pwn", name="Paiwan", _and_separator="dja", _or_separator="uri"
)
VLAX_ROMANI: Language = Language(
    code="rmy", name="Vlax Romani", _and_separator="thaj", _or_separator="vaj"
)
RUSYN: Language = Language(
    code="rue", name="Rusyn", _and_separator="і", _or_separator="або"
)
YAKUT: Language = Language(
    code="sah", name="Yakut", _and_separator="уонна", _or_separator="эбэтэр"
)
SANTALI: Language = Language(
    code="sat", name="Santali", _and_separator="ᱟᱨ", _or_separator="ᱥᱮ"
)
SICILIAN: Language = Language(
    code="scn", name="Sicilian", _and_separator="e", _or_separator="o"
)
SCOTS: Language = Language(
    code="sco", name="Scots", _and_separator="an", _or_separator="or"
)
TACHELHIT: Language = Language(
    code="shi", name="Tachelhit", _and_separator="d", _or_separator="neɣ"
)
SHAN: Language = Language(
    code="shn", name="Shan", _and_separator="လႄႈ", _or_separator="หรือ"
)
SARAIKI: Language = Language(
    code="skr", name="Saraiki", _and_separator="تے", _or_separator="یا"
)
INARI_SAMI: Language = Language(
    code="smn", name="Inari Sami", _and_separator="ja", _or_separator="teikkâ"
)
SRANAN: Language = Language(
    code="srn", name="Sranan", _and_separator="nanga", _or_separator="efu"
)
SATERLAND_FRISIAN: Language = Language(
    code="stq", name="Saterland Frisian", _and_separator="un", _or_separator="of"
)
SILESIAN: Language = Language(
    code="szl", name="Silesian", _and_separator="a", _or_separator="abo"
)
SAKIZAYA: Language = Language(
    code="szy", name="Sakizaya", _and_separator="ata", _or_separator="uduli"
)
ATAYAL: Language = Language(
    code="tay", name="Atayal", _and_separator="daha", _or_separator="ima"
)
TULU: Language = Language(
    code="tcy", name="Tulu", _and_separator="ಬೊಕ್ಕ", _or_separator="ಅತ್ತಂಡ"
)
TETUM: Language = Language(
    code="tet", name="Tetum", _and_separator="no", _or_separator="ka"
)
TALYSH: Language = Language(
    code="tly", name="Talysh", _and_separator="u", _or_separator="jo"
)
TOK_PISIN: Language = Language(
    code="tpi", name="Tok Pisin", _and_separator="na", _or_separator="o"
)
TAROKO: Language = Language(
    code="trv", name="Taroko", _and_separator="daha", _or_separator="ima"
)
TUMBUKA: Language = Language(
    code="tum", name="Tumbuka", _and_separator="na", _or_separator="panji"
)
TUVINIAN: Language = Language(
    code="tyv", name="Tuvinian", _and_separator=" болгаш", _or_separator="азы"
)
UDMURT: Language = Language(
    code="udm", name="Udmurt", _and_separator="но", _or_separator="яке"
)
VENETIAN: Language = Language(
    code="vec", name="Venetian", _and_separator="e", _or_separator="o"
)
VEPS: Language = Language(
    code="vep", name="Veps", _and_separator="da", _or_separator="vai"
)
WEST_FLEMISH: Language = Language(
    code="vls", name="West Flemish", _and_separator="en", _or_separator="of"
)
WARAY: Language = Language(
    code="war", name="Waray", _and_separator="ug", _or_separator="o"
)
WU_CHINESE: Language = Language(
    code="wuu", name="Wu Chinese", _and_separator="搭", _or_separator="或者"
)
KALMYK: Language = Language(
    code="xal", name="Kalmyk", _and_separator="болн", _or_separator="эсвл"
)
MINGRELIAN: Language = Language(
    code="xmf", name="Mingrelian", _and_separator="დო", _or_separator="ვარდა"
)
ZEELANDIC: Language = Language(
    code="zea", name="Zeelandic", _and_separator="en", _or_separator="of"
)
CANTONESE: Language = Language(
    code="yue", name="Cantonese", _and_separator="同", _or_separator="或者"
)



"""Functions related to the loading of models."""

import logging
import typing as t

from .benchmark_modules import (
    FreshEncoderModel,
    HuggingFaceEncoderModel,
    LiteLLMModel,
    VLLMModel,
)
from .enums import InferenceBackend, ModelType
from .exceptions import InvalidModel
from .logging_utils import log_once

if t.TYPE_CHECKING:
    from .benchmark_modules import BenchmarkModule
    from .data_models import BenchmarkConfig, DatasetConfig, ModelConfig


def load_model(
    model_config: "ModelConfig",
    dataset_config: "DatasetConfig",
    benchmark_config: "BenchmarkConfig",
) -> "BenchmarkModule":
    """Load a model.

    Args:
        model_config:
            The model configuration.
        dataset_config:
            The dataset configuration.
        benchmark_config:
            The benchmark configuration.

    Returns:
        The model.
    """
    log_once(f"\nLoading the model {model_config.model_id}...", level=logging.INFO)

    # The order matters; the first model type that matches will be used. For this
    # reason, they have been ordered in terms of the most common model types.
    model_class: t.Type["BenchmarkModule"]
    match (model_config.model_type, model_config.inference_backend, model_config.fresh):
        case (ModelType.GENERATIVE, InferenceBackend.VLLM, False):
            model_class = VLLMModel
        case (ModelType.ENCODER, InferenceBackend.TRANSFORMERS, False):
            model_class = HuggingFaceEncoderModel
        case (ModelType.GENERATIVE, InferenceBackend.LITELLM, False):
            model_class = LiteLLMModel
        case (ModelType.ENCODER, InferenceBackend.TRANSFORMERS, True):
            model_class = FreshEncoderModel
        case (_, _, True):
            raise InvalidModel(
                "Cannot load a freshly initialised model with the model type "
                f"{model_config.model_type!r} and inference backend "
                f"{model_config.inference_backend!r}."
            )
        case _:
            raise InvalidModel(
                f"Cannot load model with model type {model_config.model_type!r} and "
                f"inference backend {model_config.inference_backend!r}."
            )

    model = model_class(
        model_config=model_config,
        dataset_config=dataset_config,
        benchmark_config=benchmark_config,
    )

    return model



"""Tests for the `cli` module."""

from click.types import ParamType


def test_cli_param_names(cli_params: dict[str, ParamType]) -> None:
    """Test that the CLI parameters have the correct names."""
    assert set(cli_params.keys()) == {
        "model",
        "task",
        "language",
        "dataset",
        "finetuning_batch_size",
        "progress_bar",
        "raise_errors",
        "verbose",
        "save_results",
        "cache_dir",
        "api_key",
        "force",
        "device",
        "trust_remote_code",
        "clear_model_cache",
        "evaluate_test_split",
        "few_shot",
        "num_iterations",
        "api_base",
        "api_version",
        "gpu_memory_utilization",
        "attention_backend",
        "requires_safetensors",
        "generative_type",
        "custom_datasets_file",
        "download_only",
        "debug",
        "model_language",
        "dataset_language",
        "batch_size",
        "help",
    }



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Bulgarian Exams knowledge dataset and upload to HF Hub."""

import json
import tarfile
from pathlib import Path
from tempfile import TemporaryDirectory

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi

from .constants import CHOICES_MAPPING  # noqa


def main() -> None:
    """Create the Bulgarian Exams knowledge dataset and upload to HF Hub."""
    # URL to the tar.gz file
    url = "https://github.com/bgGLUE/bgglue/raw/refs/heads/main/data/exams.tar.gz"

    with TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)

        data_dir = download_dataset(url=url, temp_path=temp_path)

        train_df = load_split(file_path=data_dir / "train.jsonl")
        val_df = load_split(file_path=data_dir / "dev.jsonl")
        test_df = load_split(file_path=data_dir / "test.jsonl")

        train_df = process_split(df=train_df)
        val_df = process_split(df=val_df)
        test_df = process_split(df=test_df)

        final_train_df, final_val_df, final_test_df = create_splits(
            train_df=train_df, val_df=val_df, test_df=test_df
        )

        dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
                "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
                "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
            }
        )

        dataset_id = "EuroEval/exams-bg-mini"
        HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
        dataset.push_to_hub(dataset_id, private=True)


def download_dataset(url: str, temp_path: Path) -> Path:
    """Download the dataset.

    Args:
        url: URL to the tar.gz file.
        temp_path: Path to the temporary directory.

    Returns:
        Path to the data directory.
    """
    tar_path = temp_path / "exams.tar.gz"
    response = requests.get(url)
    response.raise_for_status()
    with open(tar_path, "wb") as f:
        f.write(response.content)

    with tarfile.open(tar_path, "r:gz") as tar:
        tar.extractall(temp_path)

    train_file = list(temp_path.glob("**/train.jsonl"))
    if not train_file:
        raise FileNotFoundError("Could not find train.jsonl in extracted files")

    data_dir = train_file[0].parent
    return data_dir


def load_split(file_path: Path) -> pd.DataFrame:
    """Load a JSONL file into a pandas DataFrame.

    Args:
        file_path: Path to the JSONL file.

    Returns:
        A DataFrame with the data.
    """
    data = []
    with open(file_path, "r", encoding="utf-8") as f:
        for line in f:
            data.append(json.loads(line))

    df = pd.DataFrame(data)
    return df


def process_split(df: pd.DataFrame) -> pd.DataFrame:
    """Process a split of the dataset.

    Args:
        df: The dataframe to process.

    Returns:
        The processed dataframe.
    """
    texts = []
    labels = []

    for _, row in df.iterrows():
        # Extract question stem
        question_stem = row["question"]["stem"]

        # Extract choices
        choices = row["question"]["choices"]
        if not len(choices) == 4:
            # Keep only the samples with 4 choices
            continue

        # Sort choices by label to ensure consistent order (A, B, C, D)
        sorted_choices = sorted(choices, key=lambda x: x["label"])

        # Build the text with choices
        choice_lines: list[str] = []
        for choice in sorted_choices:
            label = choice["label"].lower()
            text = choice["text"]
            choice_lines.append(f"{label}. {text}")

        # Get Bulgarian word for "Choices"
        choices_text = CHOICES_MAPPING.get("bg", "Възможности")

        # Construct the full text
        text = f"{question_stem}\n{choices_text}:\n" + "\n".join(choice_lines)

        # Get the correct answer label (lowercase)
        label = row.answerKey.lower()

        texts.append(text)
        labels.append(label)

    result_df = pd.DataFrame({"text": texts, "label": labels})

    # Drop duplicates based on text
    result_df = result_df.drop_duplicates(subset="text").reset_index(drop=True)

    return result_df


def create_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Create splits for the dataset.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    train_size = 1024
    test_size = 2048

    final_train_df = train_df.sample(n=train_size, random_state=4242)
    remaining_train_df = train_df.drop(final_train_df.index.tolist())
    test_df_with_remaining_train_samples = pd.concat(
        [test_df, remaining_train_df], ignore_index=True
    )

    n_missing_samples = test_size - len(test_df_with_remaining_train_samples)
    additional_val_samples = val_df.sample(n=n_missing_samples, random_state=4242)
    final_test_df = pd.concat(
        [test_df_with_remaining_train_samples, additional_val_samples],
        ignore_index=True,
    )
    final_val_df = val_df.drop(additional_val_samples.index.tolist())

    final_train_df = final_train_df.reset_index(drop=True)
    final_val_df = final_val_df.reset_index(drop=True)
    final_test_df = final_test_df.reset_index(drop=True)

    return final_train_df, final_val_df, final_test_df


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Slovene SentiNews sentiment dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import HfApi
from sklearn.utils import resample


def main() -> None:
    """Create the Slovene SentiNews sentiment dataset and upload to HF Hub."""
    # Define the dataset repository
    repo_id = "cjvt/sentinews"

    # Download the dataset
    dataset = load_dataset(repo_id, "sentence_level", split="train")
    assert isinstance(dataset, Dataset)

    # Convert to dataframe
    df = dataset.to_pandas()
    assert isinstance(df, pd.DataFrame)

    # Fix columns
    df = df.rename(columns={"content": "text", "sentiment": "label"})
    df = df[["text", "label"]]
    assert isinstance(df, pd.DataFrame)

    # Make splits
    test_size = 2048
    val_size = 256
    train_size = 1024

    # Create uniform label distribution
    df = create_uniform_label_distribution(df=df)

    # Sample final splits
    final_test_df = df.sample(n=test_size, random_state=42).reset_index(drop=True)
    df = df[~df.index.isin(final_test_df.index)]
    final_val_df = df.sample(n=val_size, random_state=42).reset_index(drop=True)
    df = df[~df.index.isin(final_val_df.index)]
    assert isinstance(df, pd.DataFrame)
    final_train_df = df.sample(n=train_size, random_state=42).reset_index(drop=True)

    assert isinstance(final_train_df, pd.DataFrame)
    assert isinstance(final_val_df, pd.DataFrame)
    assert isinstance(final_test_df, pd.DataFrame)

    # Create a dataset dictionary with custom splits
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df),
            "val": Dataset.from_pandas(final_val_df),
            "test": Dataset.from_pandas(final_test_df),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/sentinews-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset_dict.push_to_hub(dataset_id, private=True)


def create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs: list[pd.DataFrame] = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes (keep original indices!)
    balanced_df = pd.concat(resampled_dfs, ignore_index=False)

    return balanced_df


if __name__ == "__main__":
    main()



"""Tests for the `dataset_configs` module."""

from collections import defaultdict
from pathlib import Path
from typing import Generator

import pytest

from euroeval import dataset_configs as dc_module
from euroeval.data_models import DatasetConfig
from euroeval.dataset_configs import get_all_dataset_configs, get_dataset_config


class TestGetAllDatasetConfigs:
    """Tests for the `get_all_dataset_configs` function."""

    @pytest.fixture(scope="class")
    def dataset_configs(self) -> Generator[dict[str, DatasetConfig], None, None]:
        """Yields all dataset configurations."""
        yield get_all_dataset_configs(custom_datasets_file=Path("custom_datasets.py"))

    def test_dataset_configs_is_dict(
        self, dataset_configs: dict[str, DatasetConfig]
    ) -> None:
        """Test that the dataset configs are a dict."""
        assert isinstance(dataset_configs, dict)

    def test_dataset_configs_are_objects(
        self, dataset_configs: dict[str, DatasetConfig]
    ) -> None:
        """Test that the dataset configs are `DatasetConfig` objects."""
        for dataset_config in dataset_configs.values():
            assert isinstance(dataset_config, DatasetConfig)


class TestGetDatasetConfig:
    """Tests for the `get_dataset_config` function."""

    def test_get_angry_tweets_config(self) -> None:
        """Test that the angry tweets dataset config can be retrieved."""
        dataset_config = get_dataset_config(
            dataset_name="multi-wiki-qa-da",
            custom_datasets_file=Path("custom_datasets.py"),
        )
        assert dataset_config.name == "multi-wiki-qa-da"

    def test_error_when_dataset_does_not_exist(self) -> None:
        """Test that an error is raised when the dataset does not exist."""
        with pytest.raises(ValueError):
            get_dataset_config(
                dataset_name="does-not-exist",
                custom_datasets_file=Path("custom_datasets.py"),
            )


def test_no_duplicate_dataset_config_variable_names() -> None:
    """Test that there are no duplicate variable names for dataset configs."""
    # Create a mapping from language name to list of variable names for the dataset
    # configs of that language
    submodules = [
        value
        for value in dc_module.__dict__.values()
        if isinstance(value, type(dc_module))
    ]
    language_to_dataset_vars: dict[str, list[str]] = {
        submodule.__name__.split(".")[-1]: [
            var_name
            for var_name, var_value in submodule.__dict__.items()
            if isinstance(var_value, DatasetConfig)
        ]
        for submodule in submodules
    }

    # Count the number of occurences of each dataset config variable name
    dataset_variable_name_counts: dict[str, int] = defaultdict(int)
    for var_names in language_to_dataset_vars.values():
        for var_name in var_names:
            dataset_variable_name_counts[var_name] += 1

    # Raise an error if any variable name occurs more than once
    duplicate_variable_names = [
        name for name, count in dataset_variable_name_counts.items() if count > 1
    ]
    assert not duplicate_variable_names, (
        f"Duplicate dataset config variable names found: {duplicate_variable_names}. "
        "Please ensure that each dataset config variable has a unique name."
    )



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Cinexio sentiment dataset and upload to HF Hub."""

import json
import tarfile
from pathlib import Path
from tempfile import TemporaryDirectory

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi
from sklearn.utils import resample

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the Cinexio sentiment dataset and upload to HF Hub."""
    # URL to the tar.gz file
    url = "https://raw.githubusercontent.com/bgGLUE/bgglue/main/data/cinexio.tar.gz"

    with TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)

        data_dir = download_dataset(url=url, temp_path=temp_path)

        train_df = load_split(file_path=data_dir / "train.jsonl")
        val_df = load_split(file_path=data_dir / "dev.jsonl")
        test_df = load_split(file_path=data_dir / "test.jsonl")

        train_df = process_split(df=train_df)
        val_df = process_split(df=val_df)
        test_df = process_split(df=test_df)

        final_train_df, final_val_df, final_test_df = create_splits(
            train_df=train_df, val_df=val_df, test_df=test_df
        )

        dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
                "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
                "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
            }
        )

        dataset_id = "EuroEval/cinexio-mini"
        HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
        dataset.push_to_hub(dataset_id, private=True)


def download_dataset(url: str, temp_path: Path) -> Path:
    """Download the dataset.

    Args:
        url: URL to the tar.gz file.
        temp_path: Path to the temporary directory.

    Returns:
        Path to the data directory.
    """
    tar_path = temp_path / "cinexio.tar.gz"
    response = requests.get(url)
    response.raise_for_status()
    with open(tar_path, "wb") as f:
        f.write(response.content)

    with tarfile.open(tar_path, "r:gz") as tar:
        tar.extractall(temp_path)

    train_file = list(temp_path.glob("**/train.jsonl"))
    if not train_file:
        raise FileNotFoundError("Could not find train.jsonl in extracted files")

    data_dir = train_file[0].parent
    return data_dir


def load_split(file_path: Path) -> pd.DataFrame:
    """Load a JSONL file into a pandas DataFrame.

    Args:
        file_path: Path to the JSONL file.

    Returns:
        A DataFrame with the data.
    """
    data = []
    with open(file_path, "r", encoding="utf-8") as f:
        for line in f:
            data.append(json.loads(line))

    df = pd.DataFrame(data)
    return df


def process_split(df: pd.DataFrame) -> pd.DataFrame:
    """Process a split of the dataset.

    Args:
        df: The dataframe to process.

    Returns:
        The processed dataframe.
    """
    df = _map_ratings_to_labels(df=df)
    df = df.rename(columns={"Comment": "text"})
    df = df.loc[["text", "label"]]
    df = _filter_by_length(df=df)
    df = df.drop_duplicates().reset_index(drop=True)
    return df


def create_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Create splits for the dataset.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    test_size = 2048
    val_size = 256
    train_size = 1024

    train_df_uniform = _create_uniform_label_distribution(
        df=train_df, random_state=4242
    )
    train_df_remaining = train_df.loc[
        ~train_df.index.isin(train_df_uniform.index)
    ].reset_index(drop=True)

    # The original test split only contains negative samples
    test_df_with_additional_samples = pd.concat(
        [test_df, train_df_remaining], ignore_index=True
    )
    test_df_uniform = _create_uniform_label_distribution(
        df=test_df_with_additional_samples, random_state=4242
    )

    val_df_uniform = _create_uniform_label_distribution(df=val_df, random_state=4242)

    final_train_df = train_df_uniform.sample(
        n=train_size, random_state=4242
    ).reset_index(drop=True)
    final_test_df = test_df_uniform.sample(n=test_size, random_state=4242).reset_index(
        drop=True
    )
    final_val_df = val_df_uniform.sample(n=val_size, random_state=4242).reset_index(
        drop=True
    )

    return final_train_df, final_val_df, final_test_df


def _create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes (keep original indices!)
    balanced_df = pd.concat(resampled_dfs, ignore_index=False)

    return balanced_df


def _map_ratings_to_labels(df: pd.DataFrame) -> pd.DataFrame:
    """Map User_Rating to sentiment labels.

    Rating scale: 0-5 (with 0.5 increments)
    - Negative: 0, 0.5, 1.0, 1.5
    - Neutral: 2.0, 2.5, 3.0, 3.5
    - Positive: 4.0, 4.5, 5.0

    Args:
        df: DataFrame with User_Rating column.

    Returns:
        DataFrame with label column.
    """
    df = df.copy()

    # Convert User_Rating to numeric if it's a string
    df["User_Rating"] = pd.to_numeric(df["User_Rating"])

    # Map ratings to labels
    def rating_to_label(rating: float) -> str:
        if rating <= 1.5:
            return "negative"
        elif rating <= 3.5:
            return "neutral"
        else:  # rating >= 4.0
            return "positive"

    df["label"] = df["User_Rating"].apply(rating_to_label)
    return df


def _filter_by_length(df: pd.DataFrame) -> pd.DataFrame:
    """Filter dataframe by text length.

    Args:
        df: The dataframe to filter.

    Returns:
        The filtered dataframe.
    """
    new_df = df.copy()
    new_df["text_len"] = new_df["text"].str.len()
    filtered_df = new_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    return filtered_df.drop(columns=["text_len"]).reset_index(drop=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the BG-NER-BSNLP-mini NER dataset and upload it to the HF Hub."""

from ast import literal_eval

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

NER_CONVERSION_DICT = {
    0: "O",
    1: "B-PER",
    2: "I-PER",
    3: "B-ORG",
    4: "I-ORG",
    5: "B-LOC",
    6: "I-LOC",
    7: "B-MISC",
    8: "I-MISC",
}


def main() -> None:
    """Create the BG-NER-BSNLP-mini NER dataset and uploads it to the HF Hub."""
    # Define dataset ID
    repo_id = "usmiva/bg_ner_bsnlp"

    # Download the dataset
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset to a dataframe
    train_df = dataset["train"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    train_df = process(df=train_df)
    test_df = process(df=test_df)

    # Create validation split from train
    val_size = 256
    val_df = train_df.sample(n=val_size, random_state=4242)
    train_df = train_df.drop(val_df.index.tolist())

    # Create test split
    test_size = 2048
    test_df = test_df.sample(n=test_size, random_state=4242)

    # Create train split
    train_size = 1024
    train_df = train_df.sample(n=train_size, random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/bg-ner-bsnlp-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df:
            The dataframe to process.

    Returns:
        The processed dataframe.
    """
    df["ner_tags"] = df["ner_tags"].apply(literal_eval)
    df["tokens"] = df["tokens"].apply(literal_eval)

    assert all(
        len(ner_tags) == len(tokens)
        for ner_tags, tokens in zip(df["ner_tags"], df["tokens"])
    ), "The length of `ner_tags` and `tokens` are not equal in each row."

    df.rename(columns={"ner_tags": "labels"}, inplace=True)
    df["labels"] = df["labels"].map(
        lambda ner_tags: [NER_CONVERSION_DICT.get(ner_tag, "O") for ner_tag in ner_tags]
    )
    return df


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.4",
#     "pandas==2.2.0",
#     "requests==2.32.5",
# ]
# ///

"""Create the RoSent Romanian sentiment dataset and upload to HF Hub."""

import json
import os
from typing import Literal

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from dotenv import load_dotenv
from huggingface_hub import HfApi
from openai import OpenAI
from openai.types.chat import ChatCompletionUserMessageParam
from pydantic import BaseModel, Field
from sklearn.model_selection import train_test_split
from sklearn.utils import resample

CACHE_FILE = "llm_sent_cache.json"


class Sentiment(BaseModel):
    """Sentiment classification."""

    sentiment: Literal["negative", "positive"] = Field(
        description="The sentiment of the text, either 'negative' or 'positive'"
    )


def load_cache() -> dict:
    """Load cache from CACHE_FILE if it exists."""
    try:
        with open(CACHE_FILE, "r") as cache_file:
            return json.load(cache_file)
    except FileNotFoundError:
        return {}


def save_cache(cache: dict) -> None:
    """Save cache to CACHE_FILE."""
    with open(CACHE_FILE, "w") as cache_file:
        json.dump(cache, cache_file, indent=4)


label_cache = load_cache()

load_dotenv()

client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])


def main() -> None:
    """Create the RoSent Romanian sentiment dataset and upload to HF Hub."""
    # Load the dataset
    dataset_id = "dumitrescustefan/ro_sent"
    dataset = load_dataset(dataset_id)
    assert isinstance(dataset, DatasetDict)
    train_df = dataset["train"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    train_df = process(df=train_df, n_samples=1500)
    test_df = process(df=test_df, n_samples=2500)

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    train_df_final = train_df.sample(n=train_size, random_state=4242)
    train_df_remaining = train_df[~train_df.index.isin(train_df_final.index)]
    val_df_final = train_df_remaining.sample(n=val_size, random_state=4242)
    test_df_final = test_df.sample(n=test_size, random_state=4242)

    # Reset indices
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    # Create DatasetDict
    mini_dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final),
            "val": Dataset.from_pandas(val_df_final),
            "test": Dataset.from_pandas(test_df_final),
        }
    )

    # Push to HF Hub
    dataset_id = "EuroEval/ro-sent-mini"
    HfApi().delete_repo(repo_id=dataset_id, repo_type="dataset", missing_ok=True)
    mini_dataset.push_to_hub(dataset_id, private=True)


def process(df: pd.DataFrame, n_samples: int) -> pd.DataFrame:
    """Process the dataframe to ensure required columns and rename.

    Args:
        df: The input dataframe.

    Returns:
        A dataframe with the required columns and renamed.
    """
    renames = {"sentence": "text"}
    df = df.rename(columns=renames)

    # Create uniform label distribution
    df = _create_uniform_label_distribution(df=df)

    # Use train_test_split with stratify to sample n_samples
    # across the label distribution. We don't want to prompt an LLM
    # on more samples than necessary.
    df, _ = train_test_split(
        df, train_size=n_samples, random_state=4242, stratify=df["label"]
    )

    label_mapping = {0: "negative", 1: "positive"}
    df["label"] = df["label"].map(label_mapping)

    # Classify text
    df["label_llm"] = df["text"].apply(_classify_text)

    # Compare labels
    df["label_comparison"] = df.apply(_compare_labels, axis=1)

    # Keep only rows where original and LLM labels are the same
    df = df[df["label_comparison"]]

    # Keep only text and label columns
    keep_columns = ["text", "label"]
    df = df[keep_columns]

    return df


def _classify_text(text: str) -> str:
    """Classify the text using GPT-4o model.

    Args:
        text: The text to classify.

    Returns:
        The sentiment of the text, either 'negative' or 'positive'
    """
    if text in label_cache:
        return label_cache[text]

    messages = [
        ChatCompletionUserMessageParam(
            role="user",
            content=(
                f"Classify the sentiment of the following text: <text>{text}</text>. "
                "Return either 'negative' or 'positive'."
            ),
        )
    ]
    completion = client.beta.chat.completions.parse(
        model="gpt-4o", messages=messages, response_format=Sentiment
    )
    parsed_response = completion.choices[0].message.parsed
    if parsed_response is None:
        raise ValueError("Parsed response is None")
    label = parsed_response.sentiment
    label_cache[text] = label
    save_cache(cache=label_cache)
    return label


def _compare_labels(row: pd.Series) -> bool:
    """Compare the labels from the RoSent model and the original dataset.

    Args:
        row: The row to compare the labels from the RoSent model and the
            original dataset.

    Returns:
        True if labels are the same, False if they are different.
    """
    return row["label"] == row["label_llm"]


def _create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes (keep original indices!)
    balanced_df = pd.concat(resampled_dfs, ignore_index=False)

    return balanced_df


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the WikiANN NER datasets and upload them to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

LANGUAGE_CODES = ["lt", "lv", "hr", "bs", "ca", "sq"]


def main() -> None:
    """Create the WikiANN NER datasets and uploads them to the HF Hub."""
    # Load the WikiANN dataset for Latvian
    dataset_id = "unimelb-nlp/wikiann"

    for language_code in LANGUAGE_CODES:
        dataset = load_dataset(dataset_id, language_code, token=True)
        assert isinstance(dataset, DatasetDict)

        # Convert the dataset to a dataframe
        train_df = dataset["train"].to_pandas()
        val_df = dataset["validation"].to_pandas()
        test_df = dataset["test"].to_pandas()
        assert isinstance(train_df, pd.DataFrame)
        assert isinstance(val_df, pd.DataFrame)
        assert isinstance(test_df, pd.DataFrame)

        # Drop all columns except for `tokens` and `ner_tags`
        columns_to_drop = [
            col for col in train_df.columns if col not in ["tokens", "ner_tags"]
        ]
        train_df.drop(columns=columns_to_drop, inplace=True)
        val_df.drop(columns=columns_to_drop, inplace=True)
        test_df.drop(columns=columns_to_drop, inplace=True)

        # Add a `text` column
        train_df["text"] = train_df["tokens"].map(lambda tokens: " ".join(tokens))
        val_df["text"] = val_df["tokens"].map(lambda tokens: " ".join(tokens))
        test_df["text"] = test_df["tokens"].map(lambda tokens: " ".join(tokens))

        # Rename `ner_tags` to `labels`
        train_df.rename(columns={"ner_tags": "labels"}, inplace=True)
        val_df.rename(columns={"ner_tags": "labels"}, inplace=True)
        test_df.rename(columns={"ner_tags": "labels"}, inplace=True)

        # Convert the NER tags from IDs to strings
        ner_conversion_dict = {
            0: "O",
            1: "B-PER",
            2: "I-PER",
            3: "B-ORG",
            4: "I-ORG",
            5: "B-LOC",
            6: "I-LOC",
            7: "B-MISC",
            8: "I-MISC",
        }

        train_df["labels"] = train_df["labels"].map(
            lambda ner_tags: [
                ner_conversion_dict.get(ner_tag, "O") for ner_tag in ner_tags
            ]
        )
        val_df["labels"] = val_df["labels"].map(
            lambda ner_tags: [
                ner_conversion_dict.get(ner_tag, "O") for ner_tag in ner_tags
            ]
        )
        test_df["labels"] = test_df["labels"].map(
            lambda ner_tags: [
                ner_conversion_dict.get(ner_tag, "O") for ner_tag in ner_tags
            ]
        )

        # Create validation split
        val_size = 256
        val_df = val_df.sample(n=min(val_size, len(val_df)), random_state=4242)

        # Create test split
        test_size = 2048
        test_df = test_df.sample(n=min(test_size, len(test_df)), random_state=4242)

        # Create train split
        train_size = 1024
        train_df = train_df.sample(n=min(train_size, len(train_df)), random_state=4242)

        # Reset the index
        train_df = train_df.reset_index(drop=True)
        val_df = val_df.reset_index(drop=True)
        test_df = test_df.reset_index(drop=True)

        # Collect datasets in a dataset dictionary
        dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
                "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
                "test": Dataset.from_pandas(test_df, split=Split.TEST),
            }
        )

        # Push the dataset to the Hugging Face Hub
        dataset_id = f"EuroEval/wikiann-{language_code}-mini"
        HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
        dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "tqdm==4.67.1",
# ]
# ///

"""Create the NorNE-mini NER datasets and upload them to the HF Hub."""

import re
from collections import defaultdict
from typing import Dict, List, Union

import pandas as pd
import requests
from datasets import Split
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from huggingface_hub.hf_api import HfApi
from tqdm.auto import tqdm


def main() -> None:
    """Create the NorNE-mini NER datasets and uploads them to the HF Hub."""
    # Define the base download URLs
    base_urls = dict(
        nb="https://github.com/UniversalDependencies/UD_Norwegian-Bokmaal/raw/master/"
        "no_bokmaal-ud-{}.conllu",
        nn="https://github.com/UniversalDependencies/UD_Norwegian-Nynorsk/raw/master/"
        "no_nynorsk-ud-{}.conllu",
    )

    # Define conversion dict to be used with the NER tags
    ner_conversion_dict = {
        "O": "O",
        "B-LOC": "B-LOC",
        "I-LOC": "I-LOC",
        "B-PER": "B-PER",
        "I-PER": "I-PER",
        "B-ORG": "B-ORG",
        "I-ORG": "I-ORG",
        "B-MISC": "B-MISC",
        "I-MISC": "I-MISC",
        "B-GPE_LOC": "B-LOC",
        "I-GPE_LOC": "I-LOC",
        "B-GPE_ORG": "B-ORG",
        "I-GPE_ORG": "I-ORG",
        "B-PROD": "B-MISC",
        "I-PROD": "I-MISC",
        "B-DRV": "B-MISC",
        "I-DRV": "I-MISC",
        "B-EVT": "B-MISC",
        "I-EVT": "I-MISC",
    }

    # Set up the progress bar and iterate over the languages
    with tqdm(["nb", "nn"], desc="Creating NorNE-mini datasets") as pbar:
        for lang in pbar:
            # Define download URLs
            train_url = base_urls[lang].format("train")
            val_url = base_urls[lang].format("dev")
            test_url = base_urls[lang].format("test")

            # Download the data
            data = dict(
                train=requests.get(train_url).text.split("\n"),
                val=requests.get(val_url).text.split("\n"),
                test=requests.get(test_url).text.split("\n"),
            )

            # Update the progress bar description
            pbar.set_description(f"Creating NorNE datasets - {lang}")

            # Iterate over the data splits
            dfs = dict()
            for split, lines in data.items():
                # Initialise the records, data dictionary and document
                records = list()
                data_dict: Dict[str, List[Union[int, str]]] = defaultdict(list)
                doc = ""

                # Iterate over the data for the given split
                for line in lines:
                    # If we are at the first line of an entry then extract the document
                    if line.startswith("# text = "):
                        doc = re.sub("# text = ", "", line)

                    # Otherwise, if the line is a comment then ignore it
                    elif line.startswith("#"):
                        continue

                    # Otherwise, if we have reached the end of an entry then store it
                    # to the list of records and reset the data dictionary and document
                    elif line == "":
                        merged_data_dict: Dict[str, Union[str, List[Union[int, str]]]]
                        if len(data_dict["tokens"]) > 0:
                            merged_data_dict = {**data_dict, "doc": doc}
                            records.append(merged_data_dict)
                        data_dict = defaultdict(list)
                        doc = ""

                    # Otherwise we are in the middle of an entry which is not a
                    # comment, so we extract the data from the line and store it in the
                    # data dictionary
                    else:
                        data_tup = line.split("\t")
                        ner_tag = data_tup[9].replace("name=", "").split("|")[-1]
                        data_dict["ids"].append(data_tup[0])
                        data_dict["tokens"].append(data_tup[1])
                        data_dict["ner_tags"].append(ner_conversion_dict[ner_tag])

                # Convert the records to a dataframe and store it
                dfs[split] = pd.DataFrame.from_records(records)

            # Merge the dataframes
            df = pd.concat(dfs.values()).reset_index(drop=True)

            # Remove duplicates
            df = df.drop_duplicates().reset_index(drop=True)

            # Create new splits
            val_df = df.sample(n=256, random_state=4242)
            df_filtered = df[~df.index.isin(val_df.index)]
            test_df = df_filtered.sample(n=2048, random_state=4242)
            full_train_df = df_filtered[~df_filtered.index.isin(test_df.index)]
            assert isinstance(full_train_df, pd.DataFrame)
            train_df = full_train_df.sample(n=1024, random_state=4242)

            assert isinstance(train_df, pd.DataFrame)
            assert isinstance(val_df, pd.DataFrame)
            assert isinstance(test_df, pd.DataFrame)

            # Collect datasets in a dataset dictionary
            dataset = DatasetDict(
                {
                    "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
                    "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
                    "test": Dataset.from_pandas(test_df, split=Split.TEST),
                    "full_train": Dataset.from_pandas(
                        full_train_df,
                        split="full_train",  # type: ignore[bad-argument-type]
                    ),
                }
            )

            # Push the dataset to the Hugging Face Hub
            dataset_id = f"EuroEval/norne-{lang}-mini"
            HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
            dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""Utility functions related to logging."""

import datetime as dt
import logging
import os
import sys
import warnings
from io import TextIOWrapper
from types import TracebackType

import litellm
from datasets.utils import disable_progress_bars as disable_datasets_progress_bars
from evaluate import disable_progress_bar as disable_evaluate_progress_bar
from huggingface_hub.utils.tqdm import (
    disable_progress_bars as disable_hf_hub_progress_bars,
)
from termcolor import colored
from tqdm.auto import tqdm
from transformers import logging as tf_logging

from .caching_utils import cache_arguments

logger = logging.getLogger("euroeval")


def get_pbar(*tqdm_args, **tqdm_kwargs) -> tqdm:
    """Get a progress bar for vLLM with custom hard-coded arguments.

    Args:
        *tqdm_args:
            Positional arguments to pass to tqdm.
        **tqdm_kwargs:
            Additional keyword arguments to pass to tqdm.

    Returns:
        A tqdm progress bar.
    """
    tqdm_kwargs = dict(colour="yellow", ascii="—▰", leave=False) | tqdm_kwargs
    tqdm_kwargs["desc"] = colored(
        text=tqdm_kwargs.get("desc", "Processing"), color="light_yellow"
    )
    return tqdm(*tqdm_args, **tqdm_kwargs)


def log(message: str, level: int, colour: str | None = None) -> None:
    """Log a message.

    Args:
        message:
            The message to log.
        level:
            The logging level. Defaults to logging.INFO.
        colour:
            The colour to use for the message. If None, a default colour will be used
            based on the logging level.

    Raises:
        ValueError:
            If the logging level is invalid.
    """
    match level:
        case logging.DEBUG:
            message = colored(
                text=(
                    "[DEBUG] "
                    + dt.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                    + f" · {message}"
                ),
                color=colour or "light_blue",
            )
            logger.debug(message)
        case logging.INFO:
            if colour is not None:
                message = colored(text=message, color=colour)
            logger.info(message)
        case logging.WARNING:
            message = colored(text=message, color=colour or "light_red")
            logger.warning(message)
        case logging.ERROR:
            message = colored(text=message, color=colour or "red")
            logger.error(message)
        case logging.CRITICAL:
            message = colored(text=message, color=colour or "red")
            logger.critical(message)
        case _:
            raise ValueError(f"Invalid logging level: {level}")


@cache_arguments("message")
def log_once(message: str, level: int, prefix: str = "") -> None:
    """Log a message once.

    This is ensured by caching the "message" argument and only logging it the first time
    this function is called with that message.

    Args:
        message:
            The message to log.
        level:
            The logging level. Defaults to logging.INFO.
        prefix:
            A prefix to add to the message, which is not considered when determining if
            the message has been logged before.
    """
    log(message=prefix + message, level=level)


def block_terminal_output() -> None:
    """Blocks libraries from writing output to the terminal.

    This filters warnings from some libraries, sets the logging level to ERROR for some
    libraries, disabled tokeniser progress bars when using Hugging Face tokenisers, and
    disables most of the logging from the `transformers` library.
    """
    if os.getenv("FULL_LOG") == "1":
        return

    # Ignore miscellaneous warnings
    warnings.filterwarnings("ignore", category=UserWarning)
    warnings.filterwarnings("ignore", category=FutureWarning)
    warnings.filterwarnings("ignore", category=RuntimeWarning)
    logging.getLogger("absl").setLevel(logging.CRITICAL)

    # Disable matplotlib logging
    logging.getLogger("matplotlib.font_manager").setLevel(logging.CRITICAL)

    # Disable PyTorch logging
    logging.getLogger("torch.utils.cpp_extension").setLevel(logging.CRITICAL)
    warnings.filterwarnings(action="ignore", module="torch*")
    os.environ["TORCH_LOGS"] = "-all"

    # Disable huggingface_hub logging
    logging.getLogger("huggingface_hub").setLevel(logging.CRITICAL)
    disable_hf_hub_progress_bars()

    # Disable LiteLLM logging
    logging.getLogger("LiteLLM").setLevel(logging.CRITICAL)
    logging.getLogger("LiteLLM Router").setLevel(logging.CRITICAL)
    logging.getLogger("LiteLLM Proxy").setLevel(logging.CRITICAL)
    logging.getLogger("openai").setLevel(logging.CRITICAL)
    logging.getLogger("httpx").setLevel(logging.CRITICAL)
    litellm.suppress_debug_info = True  # type: ignore[bad-assignment]
    litellm.turn_off_message_logging = True

    # Disable vLLM logging
    logging.getLogger("vllm").setLevel(logging.CRITICAL)
    logging.getLogger("vllm.engine.llm_engine").setLevel(logging.CRITICAL)
    logging.getLogger("vllm.transformers_utils.tokenizer").setLevel(logging.CRITICAL)
    logging.getLogger("vllm.core.scheduler").setLevel(logging.CRITICAL)
    logging.getLogger("vllm.model_executor.weight_utils").setLevel(logging.CRITICAL)
    logging.getLogger("vllm.platforms").setLevel(logging.CRITICAL)
    logging.getLogger("mistral_common.tokens.tokenizers.tekken").setLevel(
        logging.CRITICAL
    )
    os.environ["LOG_LEVEL"] = "CRITICAL"
    os.environ["VLLM_CONFIGURE_LOGGING"] = "0"

    # Disable flashinfer logging
    os.environ["FLASHINFER_LOGGING_LEVEL"] = "CRITICAL"

    # Disable datasets logging
    logging.getLogger("datasets").setLevel(logging.CRITICAL)
    logging.getLogger("filelock").setLevel(logging.CRITICAL)
    disable_datasets_progress_bars()

    # Disable evaluate logging
    warnings.filterwarnings("ignore", module="seqeval*")
    disable_evaluate_progress_bar()

    # Disable most of the `transformers` logging
    tf_logging._default_log_level = logging.CRITICAL  # type: ignore[bad-assignment]
    tf_logging.set_verbosity(logging.CRITICAL)
    logging.getLogger("transformers.trainer").setLevel(logging.CRITICAL)
    logging.getLogger("accelerate").setLevel(logging.CRITICAL)


class no_terminal_output:
    """Context manager that suppresses all terminal output."""

    def __init__(self, disable: bool = False) -> None:
        """Initialise the context manager.

        Args:
            disable:
                If True, this context manager does nothing.
        """
        self.disable = disable
        self.devnull_file: TextIOWrapper | None = None
        self._original_stdout_fd: int | None = None
        self._original_stderr_fd: int | None = None

    def _log_windows_warning(self) -> None:
        """Log a warning about Windows not supporting blocking terminal output."""
        log_once(
            "Your operating system (probably Windows) does not support blocking "
            "terminal output, so expect more messy output - sorry!",
            level=logging.WARNING,
        )

    def __enter__(self) -> None:
        """Suppress all terminal output."""
        if self.disable:
            return

        try:
            # Save original FDs by duplicating them
            self._original_stdout_fd = os.dup(sys.stdout.fileno())
            self._original_stderr_fd = os.dup(sys.stderr.fileno())

            # Open /dev/null
            self.devnull_file = open(os.devnull, "w")

            # Redirect stdout/stderr to /dev/null
            os.dup2(self.devnull_file.fileno(), sys.stdout.fileno())
            os.dup2(self.devnull_file.fileno(), sys.stderr.fileno())

        except OSError:
            self._log_windows_warning()
            # If setup fails, clean up any resources we might have acquired
            self.__exit__(None, None, None)

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        """Re-enable terminal output."""
        if self.disable:
            return

        # Restore stdout/stderr from our saved FDs
        try:
            if self._original_stdout_fd is not None:
                os.dup2(self._original_stdout_fd, sys.stdout.fileno())
            if self._original_stderr_fd is not None:
                os.dup2(self._original_stderr_fd, sys.stderr.fileno())
        except OSError:
            self._log_windows_warning()
        finally:
            # Close the duplicated FDs we created
            if self._original_stdout_fd is not None:
                os.close(self._original_stdout_fd)
            if self._original_stderr_fd is not None:
                os.close(self._original_stderr_fd)

            # Close the /dev/null file
            if self.devnull_file is not None:
                self.devnull_file.close()


def adjust_logging_level(verbose: bool, ignore_testing: bool = False) -> int:
    """Adjust the logging level based on verbosity.

    Args:
        verbose:
            Whether to output additional output.
        ignore_testing:
            Whether to ignore the testing flag.

    Returns:
        The logging level that was set.
    """
    if hasattr(sys, "_called_from_test") and not ignore_testing:
        logging_level = logging.CRITICAL
    elif verbose:
        logging_level = logging.DEBUG
    else:
        logging_level = logging.INFO
    logger.setLevel(logging_level)
    return logging_level



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the MMLU-et dataset and upload to HF Hub."""

from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, disable_progress_bars, load_dataset
from huggingface_hub import HfApi
from sklearn.model_selection import train_test_split
from tqdm.auto import tqdm

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the MMLU-et dataset and upload to HF Hub."""
    disable_progress_bars()
    repo_id = "TalTechNLP/MMLU_et"

    # Get all the subsets of the dataset
    subsets = [
        config["config_name"]
        for config in HfApi()
        .repo_info(repo_id=repo_id, repo_type="dataset")
        .card_data.configs
    ]

    # Download the dataset
    dfs: list[pd.DataFrame] = list()
    for subset in tqdm(subsets, desc="Downloading subsets"):
        dataset = load_dataset(path=repo_id, name=subset, split="train")
        assert isinstance(dataset, Dataset)
        df = dataset.to_pandas()
        assert isinstance(df, pd.DataFrame)
        df["category"] = subset
        dfs.append(df)
    df = pd.concat(dfs, ignore_index=True)

    # Rename the columns
    df.rename(columns=dict(answer="label", question="instruction"), inplace=True)

    # Convert the labels to single lowercase letters
    df.label = df.label.map(lambda x: "abcd"[x])

    # Double check that there are exactly 4 choices per question
    assert df.choices.apply(len).eq(4).all(), "Not all questions have exactly 4 choices"

    # Split up the choices into separate columns
    df["option_a"] = df.choices.apply(lambda x: x[0])
    df["option_b"] = df.choices.apply(lambda x: x[1])
    df["option_c"] = df.choices.apply(lambda x: x[2])
    df["option_d"] = df.choices.apply(lambda x: x[3])

    # Remove the samples with overly short or long texts
    df = df[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]

    def is_repetitive(text: str) -> bool:
        """Return True if the text is repetitive."""
        max_repetitions = max(Counter(text.split()).values())
        return max_repetitions > MAX_REPETITIONS

    # Remove overly repetitive samples
    df = df[
        ~df.instruction.apply(is_repetitive)
        & ~df.option_a.apply(is_repetitive)
        & ~df.option_b.apply(is_repetitive)
        & ~df.option_c.apply(is_repetitive)
        & ~df.option_d.apply(is_repetitive)
    ]
    assert isinstance(df, pd.DataFrame)

    # Make a `text` column with all the options in it
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING['et']}:\n"
        "a. " + row.option_a.replace("\n", " ").strip() + "\n"
        "b. " + row.option_b.replace("\n", " ").strip() + "\n"
        "c. " + row.option_c.replace("\n", " ").strip() + "\n"
        "d. " + row.option_d.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Make the `label` column case-consistent with the `text` column
    df.label = df.label.str.lower()

    # Only keep the `text`, `label` and `category` columns
    df = df[["text", "label", "category"]]

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Create validation split
    val_size = 256
    traintest_arr, val_arr = train_test_split(
        df, test_size=val_size, random_state=4242, stratify=df.category
    )
    traintest_df = pd.DataFrame(traintest_arr, columns=df.columns)
    val_df = pd.DataFrame(val_arr, columns=df.columns)

    # Create test split
    test_size = 2048
    train_arr, test_arr = train_test_split(
        traintest_df,
        test_size=test_size,
        random_state=4242,
        stratify=traintest_df.category,
    )
    train_df = pd.DataFrame(train_arr, columns=df.columns)
    test_df = pd.DataFrame(test_arr, columns=df.columns)

    # Create train split
    train_size = 1024
    train_df = train_df.sample(train_size, random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/mmlu-et-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All Italian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import ITALIAN
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, MCRC, NER, RC, SENT, SUMM

### Official datasets ###

SENTIPOLC_CONFIG = DatasetConfig(
    name="sentipolc16",
    pretty_name="Sentipolc16",
    source="EuroEval/sentipolc16-mini",
    task=SENT,
    languages=[ITALIAN],
)

SCALA_IT_CONFIG = DatasetConfig(
    name="scala-it",
    pretty_name="ScaLA-it",
    source="EuroEval/scala-it",
    task=LA,
    languages=[ITALIAN],
)

MULTINERD_IT_CONFIG = DatasetConfig(
    name="multinerd-it",
    pretty_name="MultiNERD-it",
    source="EuroEval/multinerd-mini-it",
    task=NER,
    languages=[ITALIAN],
)

SQUAD_IT_CONFIG = DatasetConfig(
    name="squad-it",
    pretty_name="SQuAD-it",
    source="EuroEval/squad-it-mini",
    task=RC,
    languages=[ITALIAN],
)

ILPOST_SUM_CONFIG = DatasetConfig(
    name="ilpost-sum",
    pretty_name="IlPost-Sum",
    source="EuroEval/ilpost-sum",
    task=SUMM,
    languages=[ITALIAN],
)

MMLU_IT_CONFIG = DatasetConfig(
    name="mmlu-it",
    pretty_name="MMLU-it",
    source="EuroEval/mmlu-it-mini",
    task=KNOW,
    languages=[ITALIAN],
)

HELLASWAG_IT_CONFIG = DatasetConfig(
    name="hellaswag-it",
    pretty_name="HellaSwag-it",
    source="EuroEval/hellaswag-it-mini",
    task=COMMON_SENSE,
    languages=[ITALIAN],
)

VALEU_IT_CONFIG = DatasetConfig(
    name="valeu-it",
    pretty_name="VaLEU-it",
    source="EuroEval/european-values-it",
    task=EUROPEAN_VALUES,
    languages=[ITALIAN],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

WIKINEURAL_IT_CONFIG = DatasetConfig(
    name="wikineural-it",
    pretty_name="WikiNeural-it",
    source="EuroEval/wikineural-mini-it",
    task=NER,
    languages=[ITALIAN],
    unofficial=True,
)

BELEBELE_IT_CONFIG = DatasetConfig(
    name="belebele-it",
    pretty_name="Belebele-it",
    source="EuroEval/belebele-it-mini",
    task=MCRC,
    languages=[ITALIAN],
    unofficial=True,
)

MULTI_WIKI_QA_IT_CONFIG = DatasetConfig(
    name="multi-wiki-qa-it",
    pretty_name="MultiWikiQA-it",
    source="EuroEval/multi-wiki-qa-it-mini",
    task=RC,
    languages=[ITALIAN],
    unofficial=True,
)

GOLDENSWAG_IT_CONFIG = DatasetConfig(
    name="goldenswag-it",
    pretty_name="GoldenSwag-it",
    source="EuroEval/goldenswag-it-mini",
    task=COMMON_SENSE,
    languages=[ITALIAN],
    unofficial=True,
)

WINOGRANDE_IT_CONFIG = DatasetConfig(
    name="winogrande-it",
    pretty_name="Winogrande-it",
    source="EuroEval/winogrande-it",
    task=COMMON_SENSE,
    languages=[ITALIAN],
    _labels=["a", "b"],
    unofficial=True,
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Greek sentiment analysis dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from sklearn.utils import resample

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the Greek sentiment analysis dataset and upload it to the HF Hub."""
    # Define the source dataset
    repo_id = "DGurgurov/greek_sa"

    dataset_dict = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset_dict, DatasetDict)

    # Process splits
    train_df = process_split(dataset_dict["train"])
    val_df = process_split(dataset_dict["validation"])
    test_df = process_split(dataset_dict["test"])

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Calculate how many additional samples needed for test split
    additional_test_samples_needed = test_size - len(test_df)

    # Take additional samples from training set for test split
    additional_test_samples = train_df.sample(
        n=additional_test_samples_needed, random_state=4242
    )

    # Combine original test with additional samples from train
    final_test_df = pd.concat([test_df, additional_test_samples], ignore_index=True)

    # Remove the additional samples from train set
    remaining_train_df = train_df.loc[
        ~train_df.index.isin(additional_test_samples.index.tolist())
    ]

    # Sample final splits
    final_test_df = final_test_df.sample(n=test_size, random_state=4242).reset_index(
        drop=True
    )
    final_val_df = val_df.sample(n=val_size, random_state=4242).reset_index(drop=True)

    # The distributions are already fairly uniform in the val and test splits
    final_train_df = create_uniform_label_distribution(df=remaining_train_df)
    final_train_df = final_train_df.sample(n=train_size, random_state=4242).reset_index(
        drop=True
    )

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/greek-sa-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


def process_split(dataset: Dataset) -> pd.DataFrame:
    """Process a split of the dataset.

    Args:
        dataset: The dataset to process.

    Returns:
        A dataframe with the processed split.
    """
    df = dataset.to_pandas()
    assert isinstance(df, pd.DataFrame)
    df["label"] = df["label"].map(lambda x: {0: "negative", 1: "positive"}[x])
    df["text_len"] = df["text"].str.len()
    df = df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    df = df.drop(columns=["text_len"]).reset_index(drop=True)
    keep_columns = ["text", "label"]
    return df.loc[keep_columns]


def create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes
    balanced_df = pd.concat(resampled_dfs, ignore_index=True)

    return balanced_df


if __name__ == "__main__":
    main()



"""Tests for the `callbacks` module."""

from dataclasses import dataclass
from typing import Generator

import pytest
from datasets import Dataset
from torch.utils.data import DataLoader
from transformers.trainer_callback import TrainerControl, TrainerState
from transformers.training_args import TrainingArguments

from euroeval.callbacks import NeverLeaveProgressCallback


@dataclass
class FakeState(TrainerState):
    """Dummy state class for testing."""

    is_local_process_zero: bool = True


@dataclass
class FakeEvalDataloader(DataLoader):
    """Dummy evaluation dataloader class for testing."""

    dataset: Dataset = Dataset.from_dict({"value": [1, 2, 3]})  # type: ignore[bad-override]

    def __len__(self) -> int:
        """Return the length of the dataloader."""
        return len(self.dataset)


class TestNeverLeaveProgressCallback:
    """Tests for the `NeverLeaveProgressCallback` class."""

    @pytest.fixture(scope="class")
    def callback(self) -> Generator[NeverLeaveProgressCallback, None, None]:
        """Yields a callback."""
        yield NeverLeaveProgressCallback()

    @pytest.fixture(scope="class")
    def state(self) -> Generator[FakeState, None, None]:
        """Yields a state."""
        yield FakeState(is_local_process_zero=True)

    @pytest.fixture(scope="class")
    def eval_dataloader(self) -> Generator[FakeEvalDataloader, None, None]:
        """Yields an evaluation dataloader."""
        yield FakeEvalDataloader()

    def test_on_train_begin_initialises_not_leaving_pbar(
        self, callback: NeverLeaveProgressCallback, state: FakeState
    ) -> None:
        """Test that the `leave` attribute on the training progress bar is False."""
        assert callback.training_bar is None
        callback.on_train_begin(
            args=TrainingArguments(), state=state, control=TrainerControl()
        )
        assert callback.training_bar is not None
        assert not callback.training_bar.leave

    def test_on_prediction_step_initialises_not_leaving_pbar(
        self,
        callback: NeverLeaveProgressCallback,
        state: FakeState,
        eval_dataloader: FakeEvalDataloader,
    ) -> None:
        """Test that the `leave` attribute on the prediction progress bar is False."""
        assert callback.prediction_bar is None
        callback.on_prediction_step(
            args=TrainingArguments(),
            state=state,
            control=TrainerControl(),
            eval_dataloader=eval_dataloader,
        )
        assert callback.prediction_bar is not None
        assert not callback.prediction_bar.leave



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Slovene NER dataset from SSJ500k and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import HfApi


def main() -> None:
    """Create the Slovene NER dataset from SSJ500k and upload it to the HF Hub."""
    # Define dataset ID
    repo_id = "cjvt/ssj500k"

    # Download the dataset
    dataset = load_dataset(
        repo_id, "named_entity_recognition", split="train", trust_remote_code=True
    )
    assert isinstance(dataset, Dataset)
    df = dataset.to_pandas()
    assert isinstance(df, pd.DataFrame)

    # Process the dataframe
    df = process(df=df)

    # Define split sizes
    val_size = 256
    test_size = 2048
    train_size = 1024

    # Create validation split
    val_df = df.sample(n=val_size, random_state=42)

    # Remove validation samples from the full dataset
    temp_df = df.drop(val_df.index.tolist())

    # Create test split
    test_df = temp_df.sample(n=test_size, random_state=42)

    # Create train split
    train_df = temp_df.drop(test_df.index.tolist()).sample(
        n=train_size, random_state=42
    )

    # Reset indices for each split
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df),
            "val": Dataset.from_pandas(val_df),
            "test": Dataset.from_pandas(test_df),
        }
    )

    # Set dataset ID for the Hugging Face Hub
    dataset_id = "EuroEval/ssj500k-ner-mini"

    # Delete existing repo if needed
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push to Hugging Face Hub
    dataset_dict.push_to_hub(dataset_id, private=True)


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process dataframe to ensure correct format.

    Args:
        df (pd.DataFrame): The dataframe to process.

    Returns:
        pd.DataFrame: The processed dataframe.
    """
    df.rename(columns={"words": "tokens", "ne_tags": "labels"}, inplace=True)

    keep_columns = ["tokens", "labels"]
    df = df.loc[keep_columns]

    assert all(
        len(tokens) == len(labels) for tokens, labels in zip(df["tokens"], df["labels"])
    ), "The length of `tokens` and `labels` are not equal in each row."

    return df


if __name__ == "__main__":
    main()



"""Templates for all multiple choice tasks."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    ALBANIAN,
    BULGARIAN,
    CATALAN,
    CROATIAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    HUNGARIAN,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    ROMANIAN,
    SERBIAN,
    SLOVAK,
    SLOVENE,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language

# TODO: Missing Faroese
MULTIPLE_CHOICE_TEMPLATES: dict["Language", PromptConfig] = {
    ALBANIAN: PromptConfig(
        default_prompt_prefix="Më poshtë janë pyetje me zgjedhje të shumëfishtë "
        "(me përgjigje).",
        default_prompt_template="Pyetje: {text}\nPërgjigje: {label}",
        default_instruction_prompt="Pyetje: {text}\n\nPërgjigjuni pyetjes së "
        "mësipërme duke u përgjigjur me {labels_str}, dhe asgjë tjetër.",
        default_prompt_label_mapping="auto",
    ),
    BULGARIAN: PromptConfig(
        default_prompt_prefix="Следват въпроси с множествен избор (с отговори).",
        default_prompt_template="Въпрос: {text}\nОтговор: {label}",
        default_instruction_prompt="Въпрос: {text}\n\nОтговорете на горния въпрос "
        "като отговорите с {labels_str}, и нищо друго.",
        default_prompt_label_mapping="auto",
    ),
    CATALAN: PromptConfig(
        default_prompt_prefix="Les següents són preguntes de selecció múltiple "
        "(amb respostes).",
        default_prompt_template="Pregunta: {text}\nResposta: {label}",
        default_instruction_prompt="Pregunta: {text}\n\nRespon a la pregunta "
        "anterior utilitzant només {labels_str}, i res més.",
        default_prompt_label_mapping="auto",
    ),
    CROATIAN: PromptConfig(
        default_prompt_prefix="Sljedeća su pitanja s višestrukim izborom "
        "(s odgovorima).",
        default_prompt_template="Pitanje: {text}\nOdgovor: {label}",
        default_instruction_prompt="Pitanje: {text}\n\nOdgovorite na gornje pitanje "
        "koristeći {labels_str}, i ništa drugo.",
        default_prompt_label_mapping="auto",
    ),
    CZECH: PromptConfig(
        default_prompt_prefix="Následující jsou otázky s výběrem z více možností "
        "(s odpověďmi).",
        default_prompt_template="Otázka: {text}\nOdpověď: {label}",
        default_instruction_prompt="Otázka: {text}\n\nOdpovězte na výše uvedenou "
        "otázku pomocí {labels_str}, a nic jiného.",
        default_prompt_label_mapping="auto",
    ),
    DANISH: PromptConfig(
        default_prompt_prefix="Følgende er multiple choice spørgsmål (med svar).",
        default_prompt_template="Spørgsmål: {text}\nSvar: {label}",
        default_instruction_prompt="Spørgsmål: {text}\n\nBesvar ovenstående spørgsmål "
        "ved at svare med {labels_str}, og intet andet.",
        default_prompt_label_mapping="auto",
    ),
    GERMAN: PromptConfig(
        default_prompt_prefix="Die folgenden Fragen sind Multiple-Choice-Fragen "
        "(mit Antworten).",
        default_prompt_template="Frage: {text}\nAntwort: {label}",
        default_instruction_prompt="Frage: {text}\n\nBeantworten Sie die obige Frage "
        "mit {labels_str}, und nichts anderes.",
        default_prompt_label_mapping="auto",
    ),
    GREEK: PromptConfig(
        default_prompt_prefix="Ακολουθούν ερωτήσεις πολλαπλών επιλογών "
        "(με απαντήσεις).",
        default_prompt_template="Ερώτηση: {text}\nΑπάντηση: {label}",
        default_instruction_prompt="Ερώτηση: {text}\n\nΑπαντήστε στην παραπάνω ερώτηση "
        "χρησιμοποιώντας {labels_str}, και τίποτα άλλο.",
        default_prompt_label_mapping="auto",
    ),
    HUNGARIAN: PromptConfig(
        default_prompt_prefix="Az alábbiakban több választási lehetőséget tartalmazó "
        "kérdések találhatók (válaszokkal együtt).",
        default_prompt_template="Kérdés: {text}\nVálasz: {label}",
        default_instruction_prompt="Kérdés: {text}\n\nVálaszoljon a fenti kérdésre "
        "az elérhető lehetőségek közül {labels_str} használatával, és semmi mással.",
        default_prompt_label_mapping="auto",
    ),
    ENGLISH: PromptConfig(
        default_prompt_prefix="The following are multiple choice questions (with "
        "answers).",
        default_prompt_template="Question: {text}\nAnswer: {label}",
        default_instruction_prompt="Question: {text}\n\nAnswer the above question by "
        "replying with {labels_str}, and nothing else.",
        default_prompt_label_mapping="auto",
    ),
    SPANISH: PromptConfig(
        default_prompt_prefix="Las siguientes son preguntas de opción múltiple "
        "(con respuestas).",
        default_prompt_template="Pregunta: {text}\nRespuesta: {label}",
        default_instruction_prompt="Pregunta: {text}\n\nResponda la pregunta anterior "
        "usando solo {labels_str}, y nada más.",
        default_prompt_label_mapping="auto",
    ),
    ESTONIAN: PromptConfig(
        default_prompt_prefix="Järgnevad on vastusevariantidega küsimused (koos "
        "vastustega).",
        default_prompt_template="Küsimus: {text}\nVastus: {label}",
        default_instruction_prompt="Küsimus: {text}\n\nVasta ülaltoodud küsimusele "
        "ainult {labels_str}, ja mitte millegi muuga.",
        default_prompt_label_mapping="auto",
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_prefix="As seguintes são perguntas de escolha múltipla "
        "(com respostas).",
        default_prompt_template="Pergunta: {text}\nResposta: {label}",
        default_instruction_prompt="Pergunta: {text}\n\nResponde à pergunta "
        "acima usando só {labels_str}, e nada mais.",
        default_prompt_label_mapping="auto",
    ),
    FINNISH: PromptConfig(
        default_prompt_prefix="Seuraavat ovat monivalintakysymyksiä (vastauksineen).",
        default_prompt_template="Kysymys: {text}\nVastaus: {label}",
        default_instruction_prompt="Kysymys: {text}\n\nVastaa yllä olevaan kysymykseen "
        "käyttämällä {labels_str}, äläkä mitään muuta.",
        default_prompt_label_mapping="auto",
    ),
    FRENCH: PromptConfig(
        default_prompt_prefix="Les questions suivantes sont des questions à choix "
        "multiples (avec réponses).",
        default_prompt_template="Question: {text}\nRéponse: {label}",
        default_instruction_prompt="Question: {text}\n\nRépondez à la question "
        "ci-dessus par {labels_str}, et rien d'autre.",
        default_prompt_label_mapping="auto",
    ),
    ICELANDIC: PromptConfig(
        default_prompt_prefix="Eftirfarandi eru fjölvalsspurningar (með svörum).",
        default_prompt_template="Spurningar: {text}\nSvara: {label}",
        default_instruction_prompt="Spurningar: {text}\n\nSvaraðu eftirfarandi "
        "spurningum með {labels_str}, og engu öðru.",
        default_prompt_label_mapping="auto",
    ),
    ITALIAN: PromptConfig(
        default_prompt_prefix="Le seguenti sono domande a scelta multipla "
        "(con relative risposte).",
        default_prompt_template="Domanda: {text}\nRisposta: {label}",
        default_instruction_prompt="Domanda: {text}\n\nRispondete alla domanda "
        "precedente con {labels_str}, e nient'altro.",
        default_prompt_label_mapping="auto",
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_prefix="Toliau pateikti daugiavariančiai klausimai "
        "(su atsakymais).",
        default_prompt_template="Klausimas: {text}\nAtsakymas: {label}",
        default_instruction_prompt="Klausimas: {text}\n\nAtsakykite į aukščiau "
        "pateiktą klausimą atsakydami {labels_str}, ir nieko daugiau.",
        default_prompt_label_mapping="auto",
    ),
    LATVIAN: PromptConfig(
        default_prompt_prefix="Tālāk seko jautājumi ar vairākām atbilžu izvēlēm "
        "(ar atbildēm).",
        default_prompt_template="Jautājums: {text}\nAtbilde: {label}",
        default_instruction_prompt="Jautājums: {text}\n\nAtbildiet uz iepriekšējo "
        "jautājumu, atbildot ar {labels_str}, un nekas cits.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_prefix="Følgende er flervalgsspørsmål (med svar).",
        default_prompt_template="Spørsmål: {text}\nSvar: {label}",
        default_instruction_prompt="Spørsmål: {text}\n\nBesvar følgende spørsmål med "
        "{labels_str}, og ikke noe annet.",
        default_prompt_label_mapping="auto",
    ),
    DUTCH: PromptConfig(
        default_prompt_prefix="Hieronder staan meerkeuzevragen (met antwoorden).",
        default_prompt_template="Vraag: {text}\nAntwoord: {label}",
        default_instruction_prompt="Vraag: {text}\n\nBeantwoord de bovenstaande vraag "
        "met {labels_str}, en niets anders.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_prefix="Følgende er flervalgsspørsmål (med svar).",
        default_prompt_template="Spørsmål: {text}\nSvar: {label}",
        default_instruction_prompt="Spørsmål: {text}\n\nBesvar følgende spørsmål med "
        "{labels_str}, og ikke noe annet.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_prefix="Følgende er flervalgsspørsmål (med svar).",
        default_prompt_template="Spørsmål: {text}\nSvar: {label}",
        default_instruction_prompt="Spørsmål: {text}\n\nBesvar følgende spørsmål med "
        "{labels_str}, og ikke noe annet.",
        default_prompt_label_mapping="auto",
    ),
    ROMANIAN: PromptConfig(
        default_prompt_prefix="Următorul este un set de întrebări cu mai multe "
        "opțiuni (cu răspunsuri).",
        default_prompt_template="Întrebare: {text}\nRăspuns: {label}",
        default_instruction_prompt="Întrebare: {text}\n\nRăspundeți la următoarea "
        "întrebare folosind {labels_str},\nși nimic altceva.",
        default_prompt_label_mapping="auto",
    ),
    POLISH: PromptConfig(
        default_prompt_prefix="Poniżej znajdują się pytania wielokrotnego wyboru "
        "(z odpowiedziami).",
        default_prompt_template="Pytanie: {text}\nOdpowiedź: {label}",
        default_instruction_prompt="Pytanie: {text}\n\nOdpowiedz na powyższe pytanie, "
        "używając {labels_str} i niczego więcej.",
        default_prompt_label_mapping="auto",
    ),
    SLOVAK: PromptConfig(
        default_prompt_prefix="Nasledujú otázky s viacerými možnosťami (s odpoveďami).",
        default_prompt_template="Otázka: {text}\nOdpoveď: {label}",
        default_instruction_prompt="Otázka: {text}\n\nOdpovedzte na nasledujúcu "
        "otázku použitím {labels_str}, a nič iné.",
        default_prompt_label_mapping="auto",
    ),
    SLOVENE: PromptConfig(
        default_prompt_prefix="Naslednja so vprašanja z več možnostmi (z odgovori).",
        default_prompt_template="Vprašanje: {text}\nOdgovor: {label}",
        default_instruction_prompt="Vprašanje: {text}\n\nOdgovorite na navedeno "
        "vprašanje z uporabo {labels_str}, in nič drugega.",
        default_prompt_label_mapping="auto",
    ),
    SERBIAN: PromptConfig(
        default_prompt_prefix="Slede pitanja višestrukog izbora (sa odgovorima).",
        default_prompt_template="Pitanje: {text}\nOdgovor: {label}",
        default_instruction_prompt="Pitanje: {text}\n\nOdgovorite na navedeno "
        "pitanje koristeći {labels_str}, i ništa drugo.",
        default_prompt_label_mapping="auto",
    ),
    SWEDISH: PromptConfig(
        default_prompt_prefix="Följande är flervalsfrågor (med svar).",
        default_prompt_template="Fråga: {text}\nSvar: {label}",
        default_instruction_prompt="Fråga: {text}\n\nBesvara följande fråga med "
        "{labels_str}, och inget annat.",
        default_prompt_label_mapping="auto",
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_prefix="Нижче наведено питання з кількома варіантами "
        "відповідей (з відповідями).",
        default_prompt_template="Питання: {text}\nВідповідь: {label}",
        default_instruction_prompt="Питання: {text}\n\nДайте відповідь на наведене "
        "вище питання, використовуючи {labels_str}, і нічого іншого.",
        default_prompt_label_mapping="auto",
    ),
}



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.4",
#     "requests==2.32.5",
#     "pandas==2.2.0",
# ]
# ///

"""Create the XQuAD datasets and upload them to the HF Hub."""

import pandas as pd
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from datasets.load import load_dataset
from datasets.splits import Split
from huggingface_hub.hf_api import HfApi
from tqdm.auto import tqdm

from .constants import MAX_NUM_CHARS_IN_CONTEXT, MIN_NUM_CHARS_IN_CONTEXT

LANGUAGES = ["de", "en", "es"]


def main() -> None:
    """Create the XQuAD datasets and upload it to the HF Hub."""
    dataset_id = "google/xquad"

    for lang in tqdm(LANGUAGES):
        target_dataset_id = f"EuroEval/xquad-{lang}"

        dataset = load_dataset(
            dataset_id, name=f"xquad.{lang}", split="validation", keep_in_memory=True
        )
        assert isinstance(dataset, Dataset)

        df = dataset.to_pandas()

        # Ensure that `df` is indeed a Pandas DataFrame
        assert isinstance(df, pd.DataFrame)

        # Only work with samples where the context is not very large or small
        lengths = df.context.str.len()
        df = df[lengths.between(MIN_NUM_CHARS_IN_CONTEXT, MAX_NUM_CHARS_IN_CONTEXT)]

        # Make splits
        val_size = 64
        test_size = 512

        val_df = df.sample(n=val_size, random_state=42)
        df = df.drop(val_df.index.tolist())
        test_df = df.sample(n=test_size, random_state=42)
        train_df = df.drop(test_df.index.tolist())
        assert len(train_df) > 500, "The training set should have at least 500 samples."

        val_df = val_df.reset_index(drop=True)
        test_df = test_df.reset_index(drop=True)
        train_df = train_df.reset_index(drop=True)
        assert isinstance(train_df, pd.DataFrame)
        assert isinstance(val_df, pd.DataFrame)
        assert isinstance(test_df, pd.DataFrame)

        train = Dataset.from_pandas(train_df, split=Split.TRAIN)
        val = Dataset.from_pandas(val_df, split=Split.VALIDATION)
        test = Dataset.from_pandas(test_df, split=Split.TEST)
        assert isinstance(train, Dataset)
        assert isinstance(val, Dataset)
        assert isinstance(test, Dataset)

        # Collect datasets in a dataset dictionary
        dataset = DatasetDict({"train": train, "validation": val, "test": test})

        # Push the dataset to the Hugging Face Hub
        HfApi().delete_repo(target_dataset_id, repo_type="dataset", missing_ok=True)
        dataset.push_to_hub(target_dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the NRK-Quiz-QA-mini dataset and upload them to the HF Hub."""

import warnings
from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from pandas.errors import SettingWithCopyWarning
from sklearn.model_selection import train_test_split

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
)

warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)


def main() -> None:
    """Create the NRK-Quiz-QA-mini dataset and upload them to the HF Hub."""
    # Define the base download URL
    repo_id = "ltg/nrk_quiz_qa"

    # Download the dataset
    nb_dataset = load_dataset(path=repo_id, name="nb", token=True, split="test")
    nn_dataset = load_dataset(path=repo_id, name="nn", token=True, split="test")
    assert isinstance(nb_dataset, Dataset) and isinstance(nn_dataset, Dataset)

    # Convert the dataset to a dataframe
    nb_df = nb_dataset.to_pandas()
    nn_df = nn_dataset.to_pandas()
    assert isinstance(nb_df, pd.DataFrame) and isinstance(nn_df, pd.DataFrame)

    # Merge the datasets
    nb_df["language"] = "nb"
    nn_df["language"] = "nn"
    df = pd.concat([nb_df, nn_df], ignore_index=True)

    # Rename the columns
    df.rename(columns=dict(question="instruction", answer="label"), inplace=True)

    # Remove the samples with overly short or long texts
    df = df[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
    ]

    def is_repetitive(text: str) -> bool:
        """Return True if the text is repetitive."""
        max_repetitions = max(Counter(text.split()).values())
        return max_repetitions > MAX_REPETITIONS

    # Remove overly repetitive samples
    df = df[~df.instruction.apply(is_repetitive)]
    assert isinstance(df, pd.DataFrame)

    # We can have at most 256 quizzes, as this is the size of the validation split and
    # we are stratifying by quiz, so we keep only the top-256 quizzes by count
    quiz_counts = df.quiz.value_counts()
    top_quizzes = quiz_counts.head(256).index
    df = df[df.quiz.isin(top_quizzes)]

    # Sanity check that there are never more than 4 options
    assert df.choices.apply(lambda x: len(x["text"])).max() <= 4

    # Make a `text` column with all the options in it
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING['no']}:\n"
        + "\n".join(
            [
                f"{char}. {clean_text(text=option)}"
                for char, option in zip("abcd", row.choices["text"])
            ]
        )
        for _, row in df.iterrows()
    ]

    # Make the `label` column case-consistent with the `text` column
    df.label = df.label.str.lower()

    # Only keep the `text`, `label` and `quiz` columns
    df = df[["text", "label", "quiz"]]
    assert isinstance(df, pd.DataFrame)

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Create validation split
    val_size = 256
    traintest_arr, val_arr = train_test_split(
        df, test_size=val_size, random_state=4242, stratify=df.quiz
    )
    traintest_df = pd.DataFrame(traintest_arr, columns=df.columns)
    val_df = pd.DataFrame(val_arr, columns=df.columns)

    # Create test split
    test_size = 2048
    train_arr, test_arr = train_test_split(
        traintest_df, test_size=test_size, random_state=4242, stratify=traintest_df.quiz
    )
    train_df = pd.DataFrame(train_arr, columns=df.columns)
    test_df = pd.DataFrame(test_arr, columns=df.columns)

    # Create train split
    max_train_size = 1024
    if len(train_df) > max_train_size:
        train_df = train_df.sample(max_train_size, random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/nrk-quiz-qa-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


def clean_text(text: str) -> str:
    """Clean the text.

    Args:
        text:
            The text to be cleaned.

    Returns:
        The cleaned text.
    """
    return text.replace("\n", " ").strip()


if __name__ == "__main__":
    main()
